# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for IOS router preferences.
"""

import os
import copy
import sys
import shutil
import math
import zipfile
import logging

from gns3.qt import QtCore, QtGui
from gns3.main_window import MainWindow
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.cloud.utils import UploadFilesThread
from gns3.utils.progress_dialog import ProgressDialog
from gns3.utils.file_copy_thread import FileCopyThread

from .. import Dynamips
from ..settings import IOS_ROUTER_SETTINGS
from ..utils.decompress_ios import isIOSCompressed
from ..utils.decompress_ios_thread import DecompressIOSThread
from ..ui.ios_router_preferences_page_ui import Ui_IOSRouterPreferencesPageWidget
from ..pages.ios_router_configuration_page import IOSRouterConfigurationPage
from ..dialogs.ios_router_wizard import IOSRouterWizard


log = logging.getLogger(__name__)


class IOSRouterPreferencesPage(QtGui.QWidget, Ui_IOSRouterPreferencesPageWidget):

    """
    QWidget preference page for IOS routers.
    """

    def __init__(self):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._ios_routers = {}
        self._items = []

        self.uiNewIOSRouterPushButton.clicked.connect(self._iosRouterNewSlot)
        self.uiEditIOSRouterPushButton.clicked.connect(self._iosRouterEditSlot)
        self.uiDeleteIOSRouterPushButton.clicked.connect(self._iosRouterDeleteSlot)
        self.uiIOSRoutersTreeWidget.currentItemChanged.connect(self._iosRouterChangedSlot)
        self.uiIOSRoutersTreeWidget.itemPressed.connect(self._iosRouterPressedSlot)
        self.uiDecompressIOSPushButton.clicked.connect(self._decompressIOSSlot)

    def _iosRouterChangedSlot(self, current, previous):
        """
        Loads a selected an IOS router from the tree widget.

        :param current: current QTreeWidgetItem instance
        :param previous: ignored
        """

        if not current:
            self.uiIOSRouterInfoTreeWidget.clear()
            return

        self.uiEditIOSRouterPushButton.setEnabled(True)
        self.uiDeleteIOSRouterPushButton.setEnabled(True)
        self.uiDecompressIOSPushButton.setEnabled(True)
        key = current.data(0, QtCore.Qt.UserRole)
        ios_router = self._ios_routers[key]
        self._refreshInfo(ios_router)

    def _iosRouterNewSlot(self):
        """
        Creates a new IOS router.
        """

        wizard = IOSRouterWizard(self._ios_routers, parent=self)
        wizard.show()
        if wizard.exec_():

            ios_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=ios_settings["server"], name=ios_settings["name"])

            self._ios_routers[key] = IOS_ROUTER_SETTINGS.copy()
            self._ios_routers[key].update(ios_settings)

            if ios_settings["server"] == 'cloud':
                import logging
                log = logging.getLogger(__name__)

                log.debug(ios_settings["image"])
                # Start uploading the image to cloud files

                self._upload_image_progress_dialog = QtGui.QProgressDialog("Uploading image file {}".format(ios_settings['image']), "Cancel", 0, 0, parent=self)
                self._upload_image_progress_dialog.setWindowModality(QtCore.Qt.WindowModal)
                self._upload_image_progress_dialog.setWindowTitle("IOS image upload")
                self._upload_image_progress_dialog.show()
                try:
                    upload_thread = UploadFilesThread(
                        self,
                        cloud_settings=MainWindow.instance().cloudSettings(),
                        files_to_upload=[(
                            self._ios_routers[key]["image"],
                            'images/' + os.path.relpath(self._ios_routers[key]["image"],
                                                        self._main_window.settings().imagesDirPath())
                        )]
                    )
                    upload_thread.completed.connect(self._imageUploadComplete)
                    upload_thread.start()
                except Exception as e:
                    self._upload_image_progress_dialog.reject()
                    log.error(e)
                    QtGui.QMessageBox.critical(self, "IOS image upload", "Error uploading IOS image: {}".format(e))

            if ios_settings["platform"] == "c7200":
                self._ios_routers[key]["midplane"] = "vxr"
                self._ios_routers[key]["npe"] = "npe-400"
            else:
                self._ios_routers[key]["iomem"] = 5

            for slot_id in range(0, 7):
                slot = "slot{}".format(slot_id)
                if slot in ios_settings:
                    self._ios_routers[key][slot] = ios_settings[slot]

            for wic_id in range(0, 3):
                wic = "wic{}".format(wic_id)
                if wic in ios_settings:
                    self._ios_routers[key][wic] = ios_settings[wic]

            self._ios_routers[key].update(ios_settings)
            item = QtGui.QTreeWidgetItem(self.uiIOSRoutersTreeWidget)
            item.setText(0, self._ios_routers[key]["name"])
            item.setIcon(0, QtGui.QIcon(self._ios_routers[key]["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiIOSRoutersTreeWidget.setCurrentItem(item)

    def _imageUploadComplete(self):
        if self._upload_image_progress_dialog.wasCanceled():
            return
        self._upload_image_progress_dialog.accept()

    def _iosRouterEditSlot(self):
        """
        Edits an IOS router.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            dialog = ConfigurationDialog(ios_router["name"], ios_router, IOSRouterConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                if ios_router["name"] != item.text(0):
                    # rename the IOS router
                    new_key = "{server}:{name}".format(server=ios_router["server"], name=ios_router["name"])
                    if new_key in self._ios_routers:
                        QtGui.QMessageBox.critical(self, "IOS router", "IOS router name {} already exists for server {}".format(ios_router["name"],
                                                                                                                                ios_router["server"]))
                        ios_router["name"] = item.text(0)
                        return
                    self._ios_routers[new_key] = self._ios_routers[key]
                    del self._ios_routers[key]
                    item.setText(0, ios_router["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)

                self._refreshInfo(ios_router)

    def _iosRouterDeleteSlot(self):
        """
        Deletes an IOS router.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]

            del self._ios_routers[key]
            self.uiIOSRoutersTreeWidget.takeTopLevelItem(self.uiIOSRoutersTreeWidget.indexOfTopLevelItem(item))
            if self._ios_routers == {}:
                self.uiEditIOSRouterPushButton.setEnabled(False)
                self.uiDeleteIOSRouterPushButton.setEnabled(False)
                self.uiDecompressIOSPushButton.setEnabled(False)

    @staticmethod
    def getIOSImage(parent):
        """

        :param parent: parent widget

        :return: path to the IOS image or None
        """

        destination_directory = os.path.join(MainWindow.instance().imagesDirPath(), "IOS")
        path, _ = QtGui.QFileDialog.getOpenFileNameAndFilter(parent,
                                                             "Select an IOS image",
                                                             destination_directory,
                                                             "All files (*.*);;IOS image (*.bin *.image)",
                                                             "IOS image (*.bin *.image)")
        if not path:
            return

        if not os.access(path, os.R_OK):
            QtGui.QMessageBox.critical(parent, "IOS image", "Cannot read {}".format(path))
            return

        if sys.platform.startswith('win'):
            # Dynamips (Cygwin acutally) doesn't like non ascii paths on Windows
            try:
                path.encode('ascii')
            except UnicodeEncodeError:
                QtGui.QMessageBox.warning(parent, "IOS image", "The IOS image filename should contains only ascii (English) characters.")

        try:
            with open(path, "rb") as f:
                # read the first 7 bytes of the file.
                elf_header_start = f.read(7)
        except OSError as e:
            QtGui.QMessageBox.critical(parent, "IOS image", "Cannot read ELF magic number: {}".format(e))
            return

        # file must start with the ELF magic number, be 32-bit, big endian and have an ELF version of 1
        if elf_header_start != b'\x7fELF\x01\x02\x01':
            QtGui.QMessageBox.critical(parent, "IOS image", "Sorry, this is not a valid IOS image!")
            return

        try:
            os.makedirs(destination_directory)
        except FileExistsError:
            pass
        except OSError as e:
            QtGui.QMessageBox.critical(parent, "IOS images directory", "Could not create the IOS images directory {}: {}".format(destination_directory, e))
            return

        if isIOSCompressed(path):
            reply = QtGui.QMessageBox.question(parent, "IOS image", "Would you like to decompress this IOS image?",
                                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
            if reply == QtGui.QMessageBox.Yes:
                decompressed_image_path = os.path.join(destination_directory, os.path.basename(os.path.splitext(path)[0] + ".image"))
                thread = DecompressIOSThread(path, decompressed_image_path)
                progress_dialog = ProgressDialog(thread,
                                                 "IOS image",
                                                 "Decompressing IOS image {}...".format(os.path.basename(path)),
                                                 "Cancel", busy=True, parent=parent)
                progress_dialog.show()
                if progress_dialog.exec_() is not False:
                    path = decompressed_image_path
                thread.wait()

        if os.path.normpath(os.path.dirname(path)) != destination_directory:
            # the IOS image is not in the default images directory
            reply = QtGui.QMessageBox.question(parent,
                                               "IOS image",
                                               "Would you like to copy {} to the default images directory".format(os.path.basename(path)),
                                               QtGui.QMessageBox.Yes,
                                               QtGui.QMessageBox.No)
            if reply == QtGui.QMessageBox.Yes:
                destination_path = os.path.join(destination_directory, os.path.basename(path))
                thread = FileCopyThread(path, destination_path)
                progress_dialog = ProgressDialog(thread, "IOS image", "Copying {}".format(os.path.basename(path)), "Cancel", busy=True, parent=parent)
                thread.deleteLater()
                progress_dialog.show()
                progress_dialog.exec_()
                errors = progress_dialog.errors()
                if errors:
                    QtGui.QMessageBox.critical(parent, "IOS image", "{}".format("".join(errors)))
                else:
                    path = destination_path

        return path

    @staticmethod
    def getMinimumRequiredRAM(path):
        """
        Returns the minimum RAM required to run an IOS image.

        :param path: path to the IOS image

        :returns: minimum RAM in MB or 0 if there is an error
        """

        try:
            if isIOSCompressed(path):
                zip_file = zipfile.ZipFile(path, "r")
                decompressed_size = 0
                for zip_info in zip_file.infolist():
                    decompressed_size += zip_info.file_size
            else:
                decompressed_size = os.path.getsize(path)
        except (zipfile.BadZipFile, OSError):
            return 0

        # get the size in MB
        decompressed_size = (decompressed_size / (1000 * 1000)) + 1
        # round up to the closest multiple of 32 (step of the RAM SpinBox)
        return math.ceil(decompressed_size / 32) * 32

    def _decompressIOSSlot(self):
        """
        Slot to decompress an IOS image.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            path = ios_router["image"]
            if not os.path.isfile(path):
                QtGui.QMessageBox.critical(self, "IOS image", "IOS image file {} is does not exist".format(path))
                return
            if not isIOSCompressed(path):
                QtGui.QMessageBox.critical(self, "IOS image", "IOS image {} is not compressed".format(os.path.basename(path)))
                return

            decompressed_image_path = os.path.splitext(path)[0] + ".image"
            if os.path.isfile(decompressed_image_path):
                QtGui.QMessageBox.critical(self, "IOS image", "Decompressed IOS image {} already exist".format(os.path.basename(decompressed_image_path)))
                return

            thread = DecompressIOSThread(path, decompressed_image_path)
            progress_dialog = ProgressDialog(thread,
                                             "IOS image",
                                             "Decompressing IOS image {}...".format(path),
                                             "Cancel", busy=True, parent=self)
            progress_dialog.show()
            if progress_dialog.exec_() is not False:
                ios_router["image"] = decompressed_image_path
                self._refreshInfo(ios_router)
            thread.wait()

    def _createSectionItem(self, name):

        section_item = QtGui.QTreeWidgetItem(self.uiIOSRouterInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, ios_router):

        self.uiIOSRouterInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtGui.QTreeWidgetItem(section_item, ["Name:", ios_router["name"]])
        QtGui.QTreeWidgetItem(section_item, ["Server:", ios_router["server"]])
        QtGui.QTreeWidgetItem(section_item, ["Platform:", ios_router["platform"]])
        if ios_router["chassis"]:
            QtGui.QTreeWidgetItem(section_item, ["Chassis:", ios_router["chassis"]])
        QtGui.QTreeWidgetItem(section_item, ["Image:", ios_router["image"]])
        if ios_router["idlepc"]:
            QtGui.QTreeWidgetItem(section_item, ["Idle-PC:", ios_router["idlepc"]])
        if ios_router["startup_config"]:
            QtGui.QTreeWidgetItem(section_item, ["Startup-config:", ios_router["startup_config"]])
        if ios_router["private_config"]:
            QtGui.QTreeWidgetItem(section_item, ["Private-config:", ios_router["private_config"]])
        if ios_router["platform"] == "c7200":
            QtGui.QTreeWidgetItem(section_item, ["Midplane:", ios_router["midplane"]])
            QtGui.QTreeWidgetItem(section_item, ["NPE:", ios_router["npe"]])

        # fill out the Memories and disk section
        section_item = self._createSectionItem("Memories and disks")
        QtGui.QTreeWidgetItem(section_item, ["RAM:", "{} MiB".format(ios_router["ram"])])
        QtGui.QTreeWidgetItem(section_item, ["NVRAM:", "{} KiB".format(ios_router["nvram"])])
        if "iomem" in ios_router and ios_router["iomem"]:
            QtGui.QTreeWidgetItem(section_item, ["I/O memory:", "{}%".format(ios_router["iomem"])])
        QtGui.QTreeWidgetItem(section_item, ["PCMCIA disk0:", "{} MiB".format(ios_router["disk0"])])
        QtGui.QTreeWidgetItem(section_item, ["PCMCIA disk1:", "{} MiB".format(ios_router["disk1"])])

        # fill out the Adapters section
        section_item = self._createSectionItem("Adapters")
        for slot_id in range(0, 7):
            slot = "slot{}".format(slot_id)
            if slot in ios_router and ios_router[slot]:
                QtGui.QTreeWidgetItem(section_item, ["Slot {}:".format(slot_id), ios_router[slot]])
        if section_item.childCount() == 0:
            self.uiIOSRouterInfoTreeWidget.takeTopLevelItem(self.uiIOSRouterInfoTreeWidget.indexOfTopLevelItem(section_item))

        # fill out the WICs section
        section_item = self._createSectionItem("WICs")
        for wic_id in range(0, 3):
            wic = "wic{}".format(wic_id)
            if wic in ios_router and ios_router[wic]:
                QtGui.QTreeWidgetItem(section_item, ["WIC {}:".format(wic_id), ios_router[wic]])
        if section_item.childCount() == 0:
            self.uiIOSRouterInfoTreeWidget.takeTopLevelItem(self.uiIOSRouterInfoTreeWidget.indexOfTopLevelItem(section_item))

        self.uiIOSRouterInfoTreeWidget.expandAll()
        self.uiIOSRouterInfoTreeWidget.resizeColumnToContents(0)
        self.uiIOSRouterInfoTreeWidget.resizeColumnToContents(1)

    def _iosRouterPressedSlot(self, item, column):
        """
        Slot for item pressed.

        :param item: ignored
        :param column: ignored
        """

        if QtGui.QApplication.mouseButtons() & QtCore.Qt.RightButton:
            self._showContextualMenu()

    def _showContextualMenu(self):
        """
        Contextual menu.
        """

        menu = QtGui.QMenu()
        change_symbol_action = QtGui.QAction("Change symbol", menu)
        change_symbol_action.setIcon(QtGui.QIcon(":/icons/node_conception.svg"))
        self.connect(change_symbol_action, QtCore.SIGNAL('triggered()'), self._changeSymbolSlot)
        menu.addAction(change_symbol_action)
        menu.exec_(QtGui.QCursor.pos())

    def _changeSymbolSlot(self):
        """
        Change a symbol for an IOS router.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            dialog = SymbolSelectionDialog(self, symbol=ios_router["default_symbol"], category=ios_router["category"])
            dialog.show()
            if dialog.exec_():
                normal_symbol, selected_symbol = dialog.getSymbols()
                category = dialog.getCategory()
                item.setIcon(0, QtGui.QIcon(normal_symbol))
                ios_router["default_symbol"] = normal_symbol
                ios_router["hover_symbol"] = selected_symbol
                ios_router["category"] = category

    def loadPreferences(self):
        """
        Loads the IOS router preferences.
        """

        dynamips_module = Dynamips.instance()
        self._ios_routers = copy.deepcopy(dynamips_module.iosRouters())
        self._items.clear()

        for key, ios_router in self._ios_routers.items():
            item = QtGui.QTreeWidgetItem(self.uiIOSRoutersTreeWidget)
            item.setText(0, ios_router["name"])
            item.setIcon(0, QtGui.QIcon(ios_router["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiIOSRoutersTreeWidget.setCurrentItem(self._items[0])
            self.uiIOSRoutersTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def savePreferences(self):
        """
        Saves the IOS router preferences.
        """

        Dynamips.instance().setIOSRouters(self._ios_routers)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import platform
import struct

try:
    import raven
    RAVEN_AVAILABLE = True
except ImportError:
    # raven is not installed with deb package in order to simplify packaging
    RAVEN_AVAILABLE = False


from .version import __version__
from .servers import Servers

import logging
log = logging.getLogger(__name__)


class CrashReport:

    """
    Report crash to a third party service
    """

    DSN = "sync+https://93967ffc7e08401fa0b7578ecf08256b:8c0a5bcbc99e42f1819c6870d6362122@app.getsentry.com/38506"
    if hasattr(sys, "frozen"):
        cacert = os.path.join(os.getcwd(), "cacert.pem")
        if os.path.isfile(cacert):
            DSN += "?ca_certs={}".format(cacert)
        else:
            log.warning("The SSL certificate bundle file '{}' could not be found".format(cacert))
    _instance = None

    def __init__(self):
        self._client = None

    def captureException(self, exception, value, tb):
        if not RAVEN_AVAILABLE:
            return
        local_server = Servers.instance().localServerSettings()
        if local_server["report_errors"]:
            if self._client is None:
                self._client = raven.Client(CrashReport.DSN, release=__version__)
            self._client.tags_context({
                "os:name": platform.system(),
                "os:release": platform.release(),
                "os:win_32": " ".join(platform.win32_ver()),
                "os:mac": "{} {}".format(platform.mac_ver()[0], platform.mac_ver()[2]),
                "os:linux": " ".join(platform.linux_distribution()),
                "python:version": "{}.{}.{}".format(sys.version_info[0],
                                                    sys.version_info[1],
                                                    sys.version_info[2]),
                "python:bit": struct.calcsize("P") * 8,
                "python:encoding": sys.getdefaultencoding(),
                "python:frozen": "{}".format(hasattr(sys, "frozen"))
            })
            try:
                report = self._client.captureException((exception, value, tb))
            except Exception as e:
                log.error("Can't send crash report to Sentry: {}".format(e))
                return
            log.info("Crash report sent with event ID: {}".format(self._client.get_ident(report)))

    @classmethod
    def instance(cls):
        if cls._instance is None:
            cls._instance = CrashReport()
        return cls._instance



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/text_editor_dialog.ui'
#
# Created: Tue Dec 23 15:45:13 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_TextEditorDialog(object):

    def setupUi(self, TextEditorDialog):
        TextEditorDialog.setObjectName(_fromUtf8("TextEditorDialog"))
        TextEditorDialog.resize(457, 333)
        TextEditorDialog.setModal(True)
        self.verticalLayout = QtGui.QVBoxLayout(TextEditorDialog)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.uiTextSettingsGroupBox = QtGui.QGroupBox(TextEditorDialog)
        self.uiTextSettingsGroupBox.setObjectName(_fromUtf8("uiTextSettingsGroupBox"))
        self.gridLayout = QtGui.QGridLayout(self.uiTextSettingsGroupBox)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiColorLabel = QtGui.QLabel(self.uiTextSettingsGroupBox)
        self.uiColorLabel.setObjectName(_fromUtf8("uiColorLabel"))
        self.gridLayout.addWidget(self.uiColorLabel, 0, 0, 1, 1)
        self.uiColorPushButton = QtGui.QPushButton(self.uiTextSettingsGroupBox)
        self.uiColorPushButton.setText(_fromUtf8(""))
        self.uiColorPushButton.setObjectName(_fromUtf8("uiColorPushButton"))
        self.gridLayout.addWidget(self.uiColorPushButton, 0, 1, 1, 1)
        self.uiRotationLabel = QtGui.QLabel(self.uiTextSettingsGroupBox)
        self.uiRotationLabel.setObjectName(_fromUtf8("uiRotationLabel"))
        self.gridLayout.addWidget(self.uiRotationLabel, 1, 0, 1, 1)
        self.uiRotationSpinBox = QtGui.QSpinBox(self.uiTextSettingsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRotationSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRotationSpinBox.setSizePolicy(sizePolicy)
        self.uiRotationSpinBox.setMinimum(-360)
        self.uiRotationSpinBox.setMaximum(360)
        self.uiRotationSpinBox.setObjectName(_fromUtf8("uiRotationSpinBox"))
        self.gridLayout.addWidget(self.uiRotationSpinBox, 1, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiTextSettingsGroupBox)
        self.uiApplyTextToAllItemsCheckBox = QtGui.QCheckBox(TextEditorDialog)
        self.uiApplyTextToAllItemsCheckBox.setObjectName(_fromUtf8("uiApplyTextToAllItemsCheckBox"))
        self.verticalLayout.addWidget(self.uiApplyTextToAllItemsCheckBox)
        self.uiPlainTextEdit = QtGui.QPlainTextEdit(TextEditorDialog)
        self.uiPlainTextEdit.setObjectName(_fromUtf8("uiPlainTextEdit"))
        self.verticalLayout.addWidget(self.uiPlainTextEdit)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.uiFontPushButton = QtGui.QPushButton(TextEditorDialog)
        self.uiFontPushButton.setObjectName(_fromUtf8("uiFontPushButton"))
        self.horizontalLayout.addWidget(self.uiFontPushButton)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiButtonBox = QtGui.QDialogButtonBox(TextEditorDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtGui.QDialogButtonBox.Apply | QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName(_fromUtf8("uiButtonBox"))
        self.horizontalLayout.addWidget(self.uiButtonBox)
        self.verticalLayout.addLayout(self.horizontalLayout)
        spacerItem1 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(TextEditorDialog)
        QtCore.QObject.connect(self.uiButtonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), TextEditorDialog.accept)
        QtCore.QObject.connect(self.uiButtonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), TextEditorDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(TextEditorDialog)

    def retranslateUi(self, TextEditorDialog):
        TextEditorDialog.setWindowTitle(_translate("TextEditorDialog", "Text editor", None))
        self.uiTextSettingsGroupBox.setTitle(_translate("TextEditorDialog", "Text settings", None))
        self.uiColorLabel.setText(_translate("TextEditorDialog", "Color:", None))
        self.uiRotationLabel.setText(_translate("TextEditorDialog", "Rotation:", None))
        self.uiRotationSpinBox.setToolTip(_translate("TextEditorDialog", "Rotation can be ajusted on the scene for a selected item while\n"
                                                     "editing (notes only) with ALT and \'+\' (or P) / ALT and \'-\' (or M)", None))
        self.uiRotationSpinBox.setSuffix(_translate("TextEditorDialog", "°", None))
        self.uiApplyTextToAllItemsCheckBox.setText(_translate("TextEditorDialog", "Apply the text below to all selected items", None))
        self.uiFontPushButton.setText(_translate("TextEditorDialog", "&Select font", None))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for QEMU VM preferences.
"""

import ntpath
import os
import copy

from gns3.qt import QtCore, QtGui
from gns3.node import Node
from gns3.main_window import MainWindow
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.cloud.utils import UploadFilesThread

from .. import Qemu
from ..settings import QEMU_VM_SETTINGS
from ..ui.qemu_vm_preferences_page_ui import Ui_QemuVMPreferencesPageWidget
from ..pages.qemu_vm_configuration_page import QemuVMConfigurationPage
from ..dialogs.qemu_vm_wizard import QemuVMWizard


class QemuVMPreferencesPage(QtGui.QWidget, Ui_QemuVMPreferencesPageWidget):

    """
    QWidget preference page for QEMU VM preferences.
    """

    def __init__(self):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._qemu_vms = {}
        self._items = []

        self.uiNewQemuVMPushButton.clicked.connect(self._qemuVMNewSlot)
        self.uiEditQemuVMPushButton.clicked.connect(self._qemuVMEditSlot)
        self.uiDeleteQemuVMPushButton.clicked.connect(self._qemuVMDeleteSlot)
        self.uiQemuVMsTreeWidget.currentItemChanged.connect(self._qemuVMChangedSlot)
        self.uiQemuVMsTreeWidget.itemPressed.connect(self._qemuVMPressedSlot)

    def _createSectionItem(self, name):

        section_item = QtGui.QTreeWidgetItem(self.uiQemuVMInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, qemu_vm):

        self.uiQemuVMInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtGui.QTreeWidgetItem(section_item, ["VM name:", qemu_vm["name"]])
        QtGui.QTreeWidgetItem(section_item, ["Server:", qemu_vm["server"]])
        QtGui.QTreeWidgetItem(section_item, ["Memory:", "{} MB".format(qemu_vm["ram"])])
        if qemu_vm["qemu_path"]:
            QtGui.QTreeWidgetItem(section_item, ["QEMU binary:", os.path.basename(qemu_vm["qemu_path"])])

        # fill out the Hard disks section
        if qemu_vm["hda_disk_image"] or qemu_vm["hdb_disk_image"] or qemu_vm["hdc_disk_image"] or qemu_vm["hdd_disk_image"]:
            section_item = self._createSectionItem("Hard disks")
            if qemu_vm["hda_disk_image"]:
                QtGui.QTreeWidgetItem(section_item, ["Disk image (hda):", qemu_vm["hda_disk_image"]])
            if qemu_vm["hdb_disk_image"]:
                QtGui.QTreeWidgetItem(section_item, ["Disk image (hdb):", qemu_vm["hdb_disk_image"]])
            if qemu_vm["hdc_disk_image"]:
                QtGui.QTreeWidgetItem(section_item, ["Disk image (hdc):", qemu_vm["hdc_disk_image"]])
            if qemu_vm["hdd_disk_image"]:
                QtGui.QTreeWidgetItem(section_item, ["Disk image (hdd):", qemu_vm["hdd_disk_image"]])

        # fill out the Network section
        section_item = self._createSectionItem("Network")
        QtGui.QTreeWidgetItem(section_item, ["Adapters:", str(qemu_vm["adapters"])])
        QtGui.QTreeWidgetItem(section_item, ["Type:", qemu_vm["adapter_type"]])

        # fill out the Linux boot section
        if qemu_vm["initrd"] or qemu_vm["kernel_image"] or qemu_vm["kernel_command_line"]:
            section_item = self._createSectionItem("Linux boot")
            if qemu_vm["initrd"]:
                QtGui.QTreeWidgetItem(section_item, ["Initial RAM disk:", qemu_vm["initrd"]])
            if qemu_vm["kernel_image"]:
                QtGui.QTreeWidgetItem(section_item, ["Kernel image:", qemu_vm["kernel_image"]])
            if qemu_vm["kernel_command_line"]:
                QtGui.QTreeWidgetItem(section_item, ["Kernel command line:", qemu_vm["kernel_command_line"]])

        # performance section
        section_item = self._createSectionItem("Optimizations")
        if qemu_vm["cpu_throttling"]:
            QtGui.QTreeWidgetItem(section_item, ["CPU throttling:", "{}%".format(qemu_vm["cpu_throttling"])])
        else:
            QtGui.QTreeWidgetItem(section_item, ["CPU throttling:", "disabled"])
        QtGui.QTreeWidgetItem(section_item, ["Process priority:", qemu_vm["process_priority"]])

        # fill out the Additional options section
        if qemu_vm["options"]:
            section_item = self._createSectionItem("Additional options")
            QtGui.QTreeWidgetItem(section_item, ["Options:", qemu_vm["options"]])

        self.uiQemuVMInfoTreeWidget.expandAll()
        self.uiQemuVMInfoTreeWidget.resizeColumnToContents(0)
        self.uiQemuVMInfoTreeWidget.resizeColumnToContents(1)

    def _qemuVMChangedSlot(self, current, previous):
        """
        Loads a selected QEMU VM from the tree widget.

        :param current: current QTreeWidgetItem instance
        :param previous: ignored
        """

        if not current:
            self.uiQemuVMInfoTreeWidget.clear()
            return

        self.uiEditQemuVMPushButton.setEnabled(True)
        self.uiDeleteQemuVMPushButton.setEnabled(True)
        key = current.data(0, QtCore.Qt.UserRole)
        qemu_vm = self._qemu_vms[key]
        self._refreshInfo(qemu_vm)

    def _qemuVMNewSlot(self):
        """
        Creates a new VM.
        """

        wizard = QemuVMWizard(self._qemu_vms, parent=self)
        wizard.show()
        if wizard.exec_():

            new_vm_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=new_vm_settings["server"], name=new_vm_settings["name"])
            if key in self._qemu_vms:
                QtGui.QMessageBox.critical(self, "New QEMU VM", "VM name {} already exists".format(new_vm_settings["name"]))
                return
            self._qemu_vms[key] = QEMU_VM_SETTINGS.copy()
            self._qemu_vms[key].update(new_vm_settings)

            item = QtGui.QTreeWidgetItem(self.uiQemuVMsTreeWidget)
            item.setText(0, self._qemu_vms[key]["name"])
            item.setIcon(0, QtGui.QIcon(self._qemu_vms[key]["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiQemuVMsTreeWidget.setCurrentItem(item)

            if self._qemu_vms[key]["server"] == 'cloud':
                self._qemu_vms[key]["options"] = "-nographic"
                self._uploadImages(new_vm_settings)

    def _imageUploadComplete(self):
        if self._upload_image_progress_dialog.wasCanceled():
            return
        self._upload_image_progress_dialog.accept()

    def _uploadImages(self, qemu_vm):
        """
        Upload hard drive images to Cloud Files.
        """

        # Start uploading the image to cloud files
        self._upload_image_progress_dialog = QtGui.QProgressDialog(
            "Uploading image file(s)", "Cancel", 0, 0, parent=self)
        self._upload_image_progress_dialog.setWindowModality(QtCore.Qt.WindowModal)
        self._upload_image_progress_dialog.setWindowTitle("Qemu image upload")
        self._upload_image_progress_dialog.show()

        try:
            uploads = []
            src = qemu_vm.get("hda_disk_image", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            src = qemu_vm.get("hdb_disk_image", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            src = qemu_vm.get("hdc_disk_image", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            src = qemu_vm.get("hdd_disk_image", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            src = qemu_vm.get("initrd", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            src = qemu_vm.get("kernel_image", None)
            if src:
                _, filename = ntpath.split(src)
                dst = "images/qemu/{}".format(filename)
                uploads.append((src, dst))

            upload_thread = UploadFilesThread(self, MainWindow.instance().cloudSettings(), uploads)
            upload_thread.completed.connect(self._imageUploadComplete)
            upload_thread.start()
        except Exception as e:
            self._upload_image_progress_dialog.reject()
            import logging
            log = logging.getLogger(__name__)
            log.error(e)
            QtGui.QMessageBox.critical(self, "Qemu image upload", "Error uploading Qemu image: {}".format(e))

    def _qemuVMEditSlot(self):
        """
        Edits a QEMU VM.
        """

        item = self.uiQemuVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            qemu_vm = self._qemu_vms[key]
            dialog = ConfigurationDialog(qemu_vm["name"], qemu_vm, QemuVMConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                if qemu_vm["name"] != item.text(0):
                    new_key = "{server}:{name}".format(server=qemu_vm["server"], name=qemu_vm["name"])
                    if new_key in self._qemu_vms:
                        QtGui.QMessageBox.critical(self, "QEMU VM", "QEMU VM name {} already exists for server {}".format(qemu_vm["name"],
                                                                                                                          qemu_vm["server"]))
                        qemu_vm["name"] = item.text(0)
                        return
                    self._qemu_vms[new_key] = self._qemu_vms[key]
                    del self._qemu_vms[key]
                    item.setText(0, qemu_vm["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)

                if qemu_vm["server"] == 'cloud':
                    self._uploadImages(qemu_vm)

                self._refreshInfo(qemu_vm)

    def _qemuVMDeleteSlot(self):
        """
        Deletes a QEMU VM.
        """

        item = self.uiQemuVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            del self._qemu_vms[key]
            self.uiQemuVMsTreeWidget.takeTopLevelItem(self.uiQemuVMsTreeWidget.indexOfTopLevelItem(item))

    def _qemuVMPressedSlot(self, item, column):
        """
        Slot for item pressed.
        """

        if QtGui.QApplication.mouseButtons() & QtCore.Qt.RightButton:
            self._showContextualMenu()

    def _showContextualMenu(self):
        """
        Contextual menu.
        """

        menu = QtGui.QMenu()
        change_symbol_action = QtGui.QAction("Change symbol", menu)
        change_symbol_action.setIcon(QtGui.QIcon(":/icons/node_conception.svg"))
        self.connect(change_symbol_action, QtCore.SIGNAL('triggered()'), self._changeSymbolSlot)
        menu.addAction(change_symbol_action)
        menu.exec_(QtGui.QCursor.pos())

    def _changeSymbolSlot(self):
        """
        Change a symbol for a QEMU VM.
        """

        item = self.uiQemuVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            qemu_vm = self._qemu_vms[key]
            dialog = SymbolSelectionDialog(self, symbol=qemu_vm["default_symbol"], category=qemu_vm["category"])
            dialog.show()
            if dialog.exec_():
                normal_symbol, selected_symbol = dialog.getSymbols()
                category = dialog.getCategory()
                item.setIcon(0, QtGui.QIcon(normal_symbol))
                qemu_vm["default_symbol"] = normal_symbol
                qemu_vm["hover_symbol"] = selected_symbol
                qemu_vm["category"] = category

    def loadPreferences(self):
        """
        Loads the QEMU VM preferences.
        """

        qemu_module = Qemu.instance()
        self._qemu_vms = copy.deepcopy(qemu_module.qemuVMs())
        self._items.clear()

        for key, qemu_vm in self._qemu_vms.items():
            item = QtGui.QTreeWidgetItem(self.uiQemuVMsTreeWidget)
            item.setText(0, qemu_vm["name"])
            item.setIcon(0, QtGui.QIcon(qemu_vm["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiQemuVMsTreeWidget.setCurrentItem(self._items[0])
            self.uiQemuVMsTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def savePreferences(self):
        """
        Saves the QEMU VM preferences.
        """

        Qemu.instance().setQemuVMs(self._qemu_vms)



# -*- coding: utf-8 -*-
from ..qt import QtGui


class ChoicesSpinBox(QtGui.QSpinBox):

    """
    A custom QSpinBox that shows only values contained in `choices` iterable
    """

    def __init__(self, choices, parent=None):
        super(ChoicesSpinBox, self).__init__(parent)

        self._choices = choices
        self._current_idx = 0
        self.setRange(min(choices), max(choices))

    def stepBy(self, p_int):
        idx = self._current_idx + p_int
        if idx < 0:
            idx = 0
        elif idx >= len(self._choices):
            idx = len(self._choices) - 1

        self._current_idx = idx
        self.setValue(self._choices[idx])

    def setValue(self, p_int):
        try:
            self._current_idx = self._choices.index(p_int)
        except ValueError:
            if p_int > self.maximum():
                p_int = self.maximum()
                self._current_idx = len(self._choices) - 1
            elif p_int < self.minimum():
                p_int = self.minimum()
                self._current_idx = 0

        super(ChoicesSpinBox, self).setValue(p_int)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
"""

from PyQt4.QtCore import pyqtSignal
from PyQt4.QtCore import QThread

import ast
import logging
import os
import time
from .cloud.utils import ssh_client
from .cloud.exceptions import KeyPairExists

from .servers import Servers
from .topology import Topology


log = logging.getLogger(__name__)


class CloudBuilder(QThread):

    """
    """
    # Notify with progress amount and instance_id
    progressUpdate = pyqtSignal(object, str)

    # Notify with current state and instance_id
    stateChange = pyqtSignal(object, str)

    # Notify when instance is ready with instance_id
    buildComplete = pyqtSignal(str)

    # Notify when the instance has been created with instance and keypair
    instanceCreated = pyqtSignal(object, object)

    # Notify when the public ip is available with ip and instance_id
    instanceHasIP = pyqtSignal(str, str)

    # Notify when instance id exists with builder and instance_id
    instanceIdExists = pyqtSignal(object, str)

    def __init__(self, parent, cloud_provider, ca_dir):
        super(QThread, self).__init__(parent)
        # Store our parent so it can be passed to threads we spawn.
        self._parent = parent
        self._provider = cloud_provider
        self._ca_dir = ca_dir
        self._start_at_create = False
        self._start_at_setup = False
        self._instance = None

    def startAtCreate(self, instance_name, flavor_id, image_id):
        self._start_at_create = True
        self._instance_name = instance_name
        self._flavor_id = flavor_id
        self._image_id = image_id

    def startAtSetup(self, instance, keypair):
        self._start_at_setup = True
        self._instance = instance
        self._key_pair = keypair

    def run(self):
        try:
            log.debug('CloudBuilder.run')
            if self._start_at_create:
                log.debug('CloudBuilder._start_at_create')
                self._createInstance(self._provider, self._instance_name, self._flavor_id,
                                     self._image_id)
                log.debug('got here 3')
            if self._start_at_setup:
                log.debug('CloudBuilder start at setup')
                self._instanceCreated(self._instance, self._key_pair)
        except Exception:
            log.exception("CloudBuilder trapped an exception:")
            log.error('CloudBuilder stopped in error state.')

    def _createInstance(self, provider, name, flavor_id, image_id):
        log.debug("Creating cloud keypair with name {}".format(name))
        key_pair = None
        while key_pair is None:
            try:
                key_pair = provider.create_key_pair(name)
            except KeyPairExists:
                log.debug("Deleting old key pair with name {}.".format(name))
                self._provider.delete_key_pair_by_name(name)
            except Exception as e:
                log.debug("create_key_pair exception {}".format(e))

        log.debug("Creating cloud server with name {}".format(name))
        instance = None
        while instance is None:
            try:
                instance = self._provider.create_instance(name, flavor_id, image_id, key_pair)
            except Exception as e:
                log.debug("create_instance exception {}".format(e))
        log.debug("Cloud server {} created".format(name))
        self._instanceCreated(instance, key_pair)

    def _instanceCreated(self, instance, key_pair):
        log.debug('CloudBuilder._instanceCreated {}'.format(instance.id))
        self._instance = instance
        self._instance_id = instance.id
        self._key_pair = key_pair
        self.instanceIdExists.emit(self, instance.id)
        self.instanceCreated.emit(instance, key_pair)
        self._waitForPublicIP()

    def _waitForPublicIP(self):
        public_ip = None
        while public_ip is None:
            time.sleep(10)
            try:
                instance = self._provider.get_instance(self._instance)
                # Look for public ip address
                for ip in instance.public_ips:
                    # Don't use the ipv6 address
                    if ':' not in ip:
                        public_ip = ip
                        break
            except Exception as e:
                log.debug('list_instances error: {}'.format(e))

        # updated info, keep it.
        self._instance = instance
        self._public_ip = public_ip
        self.instanceHasIP.emit(self._public_ip, self._instance.id)
        time.sleep(60)
        self._startGNS3Server(1800)

    def _startGNS3Server(self, dead_time):
        commands = '''
DEBIAN_FRONTEND=noninteractive dpkg --configure -a
DEBIAN_FRONTEND=noninteractive dpkg --add-architecture i386
DEBIAN_FRONTEND=noninteractive apt-get -y update
DEBIAN_FRONTEND=noninteractive apt-get -o Dpkg::Options::="--force-confnew" --force-yes -fuy dist-upgrade
DEBIAN_FRONTEND=noninteractive apt-get -y install git python3-setuptools python3-netifaces python3-pip python3-zmq dynamips qemu-system
DEBIAN_FRONTEND=noninteractive apt-get -y install libc6:i386 libstdc++6:i386 libssl1.0.0:i386
ln -s /lib/i386-linux-gnu/libcrypto.so.1.0.0 /lib/i386-linux-gnu/libcrypto.so.4
mkdir -p /opt/gns3
cd /opt/gns3; git clone https://github.com/planctechnologies/gns3-server.git
cd /opt/gns3/gns3-server; git checkout dev; git pull
cd /opt/gns3/gns3-server; pip3 install -r dev-requirements.txt
cd /opt/gns3/gns3-server; python3 ./setup.py install
ln -sf /usr/bin/dynamips /usr/local/bin/dynamips
wget 'https://github.com/GNS3/iouyap/releases/download/0.95/iouyap.tar.gz'
tar xzf iouyap.tar.gz -C /usr/local/bin
python -c 'import struct; open("/etc/hostid", "w").write(struct.pack("i", 00000000))'
hostname gns3-iouvm # set hostname for iou
wget 'http://downloads.sourceforge.net/project/vpcs/0.6/vpcs_0.6_Linux64'
cp vpcs_0.6_Linux64 /usr/local/bin/vpcs
chmod a+x /usr/local/bin/vpcs
killall python3 gns3server gns3dms
'''

        def exec_command(client, cmd, wait_time=-1):

            cmd += '; exit $?'

            stdout_data = b''
            stderr_data = b''

            log.debug('cmd: {}'.format(cmd))
            # Send the command (non-blocking)
            stdin, stdout, stderr = client.exec_command(cmd)

            # Wait for the command to terminate
            wait = int(wait_time)
            while not stdout.channel.exit_status_ready() and wait != 0:
                time.sleep(1)
                wait -= 1

            stdout_data = stdout.read()
            stderr_data = stderr.read()
            log.debug('exit status: {}'.format(stdout.channel.exit_status))
            log.debug('stdout: {}'.format(stdout_data.decode('utf-8')))
            log.debug('stderr: {}'.format(stderr_data.decode('utf-8')))
            return stdout_data, stderr_data

        # We might be attempting a connection before the instance is fully booted, so retry
        # when the ssh connection fails.
        ssh_connected = False
        response = None
        while not ssh_connected:
            with ssh_client(self._public_ip, self._key_pair.private_key) as client:
                if client is None:
                    time.sleep(1)
                    continue
                ssh_connected = True

                for cmd in [l for l in commands.splitlines() if l.strip()]:
                    exec_command(client, cmd)

                data = {
                    'instance_id': self._instance_id,
                    'cloud_user_name': self._provider.username,
                    'cloud_api_key': self._provider.api_key,
                    'cloud_region': self._provider.region,
                    'dead_time': dead_time,
                }
                # TODO: Properly escape the data portion of the command line
                start_cmd = '/usr/bin/python3 /opt/gns3/gns3-server/gns3server/start_server.py -d -v --ip={} --data="{}" 2>/tmp/gns3-stderr.log'.format(self._public_ip, data)
                stdout, stderr = exec_command(client, start_cmd, wait_time=15)
                response = stdout.decode('utf-8')

        log.debug(response)
        data = ast.literal_eval(response)
        # TODO: have the server return the port it is running on
        port = 8000

        username = data['WEB_USERNAME']
        password = data['WEB_PASSWORD']

        ssl_cert = ''.join(data['SSL_CRT'])
        ca_filename = 'cloud_server_{}.crt'.format(self._public_ip)
        ca_dir = self._ca_dir
        ca_file = os.path.join(ca_dir, ca_filename)
        try:
            os.makedirs(ca_dir)
        except FileExistsError:
            pass
        with open(ca_file, 'wb') as ca_fh:
            ca_fh.write(ssl_cert.encode('utf-8'))

        topology = Topology.instance()
        top_instance = topology.getInstance(self._instance_id)
        top_instance.set_later_attributes(self._public_ip, port, ssl_cert, ca_file)

        servers = Servers.instance()
        server = servers.getCloudServer(self._public_ip, port, ca_file, username, password,
                                        self._key_pair.private_key, self._instance_id)
        servers.save()
        log.debug('Cloud server gns3server started.')
        self.buildComplete.emit(self._instance_id)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for VirtualBox VM preferences.
"""

import copy

from gns3.qt import QtCore, QtGui
from gns3.main_window import MainWindow
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.dialogs.configuration_dialog import ConfigurationDialog

from .. import VirtualBox
from ..settings import VBOX_VM_SETTINGS
from ..ui.virtualbox_vm_preferences_page_ui import Ui_VirtualBoxVMPreferencesPageWidget
from ..pages.virtualbox_vm_configuration_page import VirtualBoxVMConfigurationPage
from ..dialogs.virtualbox_vm_wizard import VirtualBoxVMWizard


class VirtualBoxVMPreferencesPage(QtGui.QWidget, Ui_VirtualBoxVMPreferencesPageWidget):

    """
    QWidget preference page for VirtualBox VM preferences.
    """

    def __init__(self):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._virtualbox_vms = {}
        self._items = []

        self.uiNewVirtualBoxVMPushButton.clicked.connect(self._vboxVMNewSlot)
        self.uiEditVirtualBoxVMPushButton.clicked.connect(self._vboxVMEditSlot)
        self.uiDeleteVirtualBoxVMPushButton.clicked.connect(self._vboxVMDeleteSlot)
        self.uiVirtualBoxVMsTreeWidget.currentItemChanged.connect(self._vboxVMChangedSlot)
        self.uiVirtualBoxVMsTreeWidget.itemPressed.connect(self._vboxVMPressedSlot)

    def _vboxVMChangedSlot(self, current, previous):
        """
        Loads a selected VirtualBox VM from the tree widget.

        :param current: current QTreeWidgetItem instance
        :param previous: ignored
        """

        if not current:
            self.uiVirtualBoxVMInfoTreeWidget.clear()
            return

        self.uiEditVirtualBoxVMPushButton.setEnabled(True)
        self.uiDeleteVirtualBoxVMPushButton.setEnabled(True)
        key = current.data(0, QtCore.Qt.UserRole)
        vbox_vm = self._virtualbox_vms[key]
        self._refreshInfo(vbox_vm)

    def _createSectionItem(self, name):

        section_item = QtGui.QTreeWidgetItem(self.uiVirtualBoxVMInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, vbox_vm):

        self.uiVirtualBoxVMInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtGui.QTreeWidgetItem(section_item, ["VM name:", vbox_vm["vmname"]])
        QtGui.QTreeWidgetItem(section_item, ["RAM:", str(vbox_vm["ram"])])
        QtGui.QTreeWidgetItem(section_item, ["Server:", vbox_vm["server"]])
        QtGui.QTreeWidgetItem(section_item, ["Remote console enabled:", "{}".format(vbox_vm["enable_remote_console"])])
        QtGui.QTreeWidgetItem(section_item, ["Headless mode enabled:", "{}".format(vbox_vm["headless"])])
        QtGui.QTreeWidgetItem(section_item, ["Linked base VM:", "{}".format(vbox_vm["linked_base"])])

        # fill out the Network section
        section_item = self._createSectionItem("Network")
        QtGui.QTreeWidgetItem(section_item, ["Adapters:", str(vbox_vm["adapters"])])
        QtGui.QTreeWidgetItem(section_item, ["Use any adapter:", "{}".format(vbox_vm["use_any_adapter"])])
        QtGui.QTreeWidgetItem(section_item, ["Type:", vbox_vm["adapter_type"]])

        self.uiVirtualBoxVMInfoTreeWidget.expandAll()
        self.uiVirtualBoxVMInfoTreeWidget.resizeColumnToContents(0)
        self.uiVirtualBoxVMInfoTreeWidget.resizeColumnToContents(1)

    def _vboxVMNewSlot(self):
        """
        Creates a new VirtualBox VM.
        """

        wizard = VirtualBoxVMWizard(self._virtualbox_vms, parent=self)
        wizard.show()
        if wizard.exec_():

            new_vm_settings = wizard.getSettings()
            key = "{server}:{vmname}".format(server=new_vm_settings["server"], vmname=new_vm_settings["vmname"])
            self._virtualbox_vms[key] = VBOX_VM_SETTINGS.copy()
            self._virtualbox_vms[key].update(new_vm_settings)

            item = QtGui.QTreeWidgetItem(self.uiVirtualBoxVMsTreeWidget)
            item.setText(0, self._virtualbox_vms[key]["vmname"])
            item.setIcon(0, QtGui.QIcon(self._virtualbox_vms[key]["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiVirtualBoxVMsTreeWidget.setCurrentItem(item)

    def _vboxVMEditSlot(self):
        """
        Edits a VirtualBox VM.
        """

        item = self.uiVirtualBoxVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            vbox_vm = self._virtualbox_vms[key]
            dialog = ConfigurationDialog(vbox_vm["vmname"], vbox_vm, VirtualBoxVMConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                if vbox_vm["vmname"] != item.text(0):
                    new_key = "{server}:{vmname}".format(server=vbox_vm["server"], name=vbox_vm["vmname"])
                    if new_key in self._virtualbox_vms:
                        QtGui.QMessageBox.critical(self, "VirtualBox VM", "VirtualBox VM name {} already exists for server {}".format(vbox_vm["vmname"],
                                                                                                                                      vbox_vm["server"]))
                        vbox_vm["vmname"] = item.text(0)
                        return
                    self._virtualbox_vms[new_key] = self._virtualbox_vms[key]
                    del self._virtualbox_vms[key]
                    item.setText(0, vbox_vm["vmname"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)
                self._refreshInfo(vbox_vm)

    def _vboxVMDeleteSlot(self):
        """
        Deletes a VirtualBox VM.
        """

        item = self.uiVirtualBoxVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            del self._virtualbox_vms[key]
            self.uiVirtualBoxVMsTreeWidget.takeTopLevelItem(self.uiVirtualBoxVMsTreeWidget.indexOfTopLevelItem(item))

    def _vboxVMPressedSlot(self, item, column):
        """
        Slot for item pressed.

        :param item: ignored
        :param column: ignored
        """

        if QtGui.QApplication.mouseButtons() & QtCore.Qt.RightButton:
            self._showContextualMenu()

    def _showContextualMenu(self):
        """
        Contextual menu.
        """

        menu = QtGui.QMenu()
        change_symbol_action = QtGui.QAction("Change symbol", menu)
        change_symbol_action.setIcon(QtGui.QIcon(":/icons/node_conception.svg"))
        self.connect(change_symbol_action, QtCore.SIGNAL('triggered()'), self._changeSymbolSlot)
        menu.addAction(change_symbol_action)
        menu.exec_(QtGui.QCursor.pos())

    def _changeSymbolSlot(self):
        """
        Change a symbol for a VirtualBox VM.
        """

        item = self.uiVirtualBoxVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            vbox_vm = self._virtualbox_vms[key]
            dialog = SymbolSelectionDialog(self, symbol=vbox_vm["default_symbol"], category=vbox_vm["category"])
            dialog.show()
            if dialog.exec_():
                normal_symbol, selected_symbol = dialog.getSymbols()
                category = dialog.getCategory()
                item.setIcon(0, QtGui.QIcon(normal_symbol))
                vbox_vm["default_symbol"] = normal_symbol
                vbox_vm["hover_symbol"] = selected_symbol
                vbox_vm["category"] = category

    def loadPreferences(self):
        """
        Loads the VirtualBox VM preferences.
        """

        vbox_module = VirtualBox.instance()
        self._virtualbox_vms = copy.deepcopy(vbox_module.virtualBoxVMs())
        self._items.clear()

        for key, vbox_vm in self._virtualbox_vms.items():
            item = QtGui.QTreeWidgetItem(self.uiVirtualBoxVMsTreeWidget)
            item.setText(0, vbox_vm["vmname"])
            item.setIcon(0, QtGui.QIcon(vbox_vm["default_symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiVirtualBoxVMsTreeWidget.setCurrentItem(self._items[0])
            self.uiVirtualBoxVMsTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def savePreferences(self):
        """
        Saves the VirtualBox VM preferences.
        """

        # self._vboxVMSaveSlot()
        VirtualBox.instance().setVirtualBoxVMs(self._virtualbox_vms)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/dynamips/ui/ios_router_configuration_page.ui'
#
# Created: Sat Mar 14 16:29:27 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_iosRouterConfigPageWidget(object):
    def setupUi(self, iosRouterConfigPageWidget):
        iosRouterConfigPageWidget.setObjectName(_fromUtf8("iosRouterConfigPageWidget"))
        iosRouterConfigPageWidget.resize(449, 491)
        self.vboxlayout = QtGui.QVBoxLayout(iosRouterConfigPageWidget)
        self.vboxlayout.setObjectName(_fromUtf8("vboxlayout"))
        self.uiTabWidget = QtGui.QTabWidget(iosRouterConfigPageWidget)
        self.uiTabWidget.setObjectName(_fromUtf8("uiTabWidget"))
        self.uiGeneralPageWidget = QtGui.QWidget()
        self.uiGeneralPageWidget.setObjectName(_fromUtf8("uiGeneralPageWidget"))
        self.gridLayout_2 = QtGui.QGridLayout(self.uiGeneralPageWidget)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.uiNameLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiNameLabel.setObjectName(_fromUtf8("uiNameLabel"))
        self.gridLayout_2.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtGui.QLineEdit(self.uiGeneralPageWidget)
        self.uiNameLineEdit.setObjectName(_fromUtf8("uiNameLineEdit"))
        self.gridLayout_2.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiPlatformLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformLabel.setObjectName(_fromUtf8("uiPlatformLabel"))
        self.gridLayout_2.addWidget(self.uiPlatformLabel, 1, 0, 1, 1)
        self.uiPlatformTextLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformTextLabel.setText(_fromUtf8(""))
        self.uiPlatformTextLabel.setObjectName(_fromUtf8("uiPlatformTextLabel"))
        self.gridLayout_2.addWidget(self.uiPlatformTextLabel, 1, 1, 1, 1)
        self.uiChassisLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiChassisLabel.setObjectName(_fromUtf8("uiChassisLabel"))
        self.gridLayout_2.addWidget(self.uiChassisLabel, 2, 0, 1, 1)
        self.uiChassisTextLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiChassisTextLabel.setText(_fromUtf8(""))
        self.uiChassisTextLabel.setObjectName(_fromUtf8("uiChassisTextLabel"))
        self.gridLayout_2.addWidget(self.uiChassisTextLabel, 2, 1, 1, 1)
        self.uiIOSImageLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiIOSImageLabel.setObjectName(_fromUtf8("uiIOSImageLabel"))
        self.gridLayout_2.addWidget(self.uiIOSImageLabel, 3, 0, 1, 1)
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.uiIOSImageLineEdit = QtGui.QLineEdit(self.uiGeneralPageWidget)
        self.uiIOSImageLineEdit.setObjectName(_fromUtf8("uiIOSImageLineEdit"))
        self.horizontalLayout_5.addWidget(self.uiIOSImageLineEdit)
        self.uiIOSImageToolButton = QtGui.QToolButton(self.uiGeneralPageWidget)
        self.uiIOSImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOSImageToolButton.setObjectName(_fromUtf8("uiIOSImageToolButton"))
        self.horizontalLayout_5.addWidget(self.uiIOSImageToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_5, 3, 1, 1, 1)
        self.uiStartupConfigLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiStartupConfigLabel.setObjectName(_fromUtf8("uiStartupConfigLabel"))
        self.gridLayout_2.addWidget(self.uiStartupConfigLabel, 4, 0, 1, 1)
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.uiStartupConfigLineEdit = QtGui.QLineEdit(self.uiGeneralPageWidget)
        self.uiStartupConfigLineEdit.setObjectName(_fromUtf8("uiStartupConfigLineEdit"))
        self.horizontalLayout_4.addWidget(self.uiStartupConfigLineEdit)
        self.uiStartupConfigToolButton = QtGui.QToolButton(self.uiGeneralPageWidget)
        self.uiStartupConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiStartupConfigToolButton.setObjectName(_fromUtf8("uiStartupConfigToolButton"))
        self.horizontalLayout_4.addWidget(self.uiStartupConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 4, 1, 1, 1)
        self.uiPrivateConfigLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiPrivateConfigLabel.setObjectName(_fromUtf8("uiPrivateConfigLabel"))
        self.gridLayout_2.addWidget(self.uiPrivateConfigLabel, 5, 0, 1, 1)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.uiPrivateConfigLineEdit = QtGui.QLineEdit(self.uiGeneralPageWidget)
        self.uiPrivateConfigLineEdit.setObjectName(_fromUtf8("uiPrivateConfigLineEdit"))
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigLineEdit)
        self.uiPrivateConfigToolButton = QtGui.QToolButton(self.uiGeneralPageWidget)
        self.uiPrivateConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiPrivateConfigToolButton.setObjectName(_fromUtf8("uiPrivateConfigToolButton"))
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 5, 1, 1, 1)
        self.uiConsolePortLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiConsolePortLabel.setObjectName(_fromUtf8("uiConsolePortLabel"))
        self.gridLayout_2.addWidget(self.uiConsolePortLabel, 6, 0, 1, 1)
        self.uiConsolePortSpinBox = QtGui.QSpinBox(self.uiGeneralPageWidget)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName(_fromUtf8("uiConsolePortSpinBox"))
        self.gridLayout_2.addWidget(self.uiConsolePortSpinBox, 6, 1, 1, 1)
        self.uiAuxPortLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiAuxPortLabel.setObjectName(_fromUtf8("uiAuxPortLabel"))
        self.gridLayout_2.addWidget(self.uiAuxPortLabel, 7, 0, 1, 1)
        self.uiAuxPortSpinBox = QtGui.QSpinBox(self.uiGeneralPageWidget)
        self.uiAuxPortSpinBox.setMaximum(65535)
        self.uiAuxPortSpinBox.setObjectName(_fromUtf8("uiAuxPortSpinBox"))
        self.gridLayout_2.addWidget(self.uiAuxPortSpinBox, 7, 1, 1, 1)
        self.uiMidplaneLabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiMidplaneLabel.setObjectName(_fromUtf8("uiMidplaneLabel"))
        self.gridLayout_2.addWidget(self.uiMidplaneLabel, 8, 0, 1, 1)
        self.uiMidplaneComboBox = QtGui.QComboBox(self.uiGeneralPageWidget)
        self.uiMidplaneComboBox.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMidplaneComboBox.sizePolicy().hasHeightForWidth())
        self.uiMidplaneComboBox.setSizePolicy(sizePolicy)
        self.uiMidplaneComboBox.setObjectName(_fromUtf8("uiMidplaneComboBox"))
        self.gridLayout_2.addWidget(self.uiMidplaneComboBox, 8, 1, 1, 1)
        self.uiNPELabel = QtGui.QLabel(self.uiGeneralPageWidget)
        self.uiNPELabel.setObjectName(_fromUtf8("uiNPELabel"))
        self.gridLayout_2.addWidget(self.uiNPELabel, 9, 0, 1, 1)
        self.uiNPEComboBox = QtGui.QComboBox(self.uiGeneralPageWidget)
        self.uiNPEComboBox.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNPEComboBox.sizePolicy().hasHeightForWidth())
        self.uiNPEComboBox.setSizePolicy(sizePolicy)
        self.uiNPEComboBox.setObjectName(_fromUtf8("uiNPEComboBox"))
        self.gridLayout_2.addWidget(self.uiNPEComboBox, 9, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(263, 151, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 10, 1, 1, 1)
        self.uiTabWidget.addTab(self.uiGeneralPageWidget, _fromUtf8(""))
        self.uiMemoriesPageWidget = QtGui.QWidget()
        self.uiMemoriesPageWidget.setObjectName(_fromUtf8("uiMemoriesPageWidget"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.uiMemoriesPageWidget)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.uiMemoriesGroupBox = QtGui.QGroupBox(self.uiMemoriesPageWidget)
        self.uiMemoriesGroupBox.setObjectName(_fromUtf8("uiMemoriesGroupBox"))
        self.gridLayout_5 = QtGui.QGridLayout(self.uiMemoriesGroupBox)
        self.gridLayout_5.setObjectName(_fromUtf8("gridLayout_5"))
        self.uiRamLabel = QtGui.QLabel(self.uiMemoriesGroupBox)
        self.uiRamLabel.setObjectName(_fromUtf8("uiRamLabel"))
        self.gridLayout_5.addWidget(self.uiRamLabel, 0, 0, 1, 1)
        self.uiRamSpinBox = QtGui.QSpinBox(self.uiMemoriesGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRamSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRamSpinBox.setSizePolicy(sizePolicy)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setSingleStep(32)
        self.uiRamSpinBox.setProperty("value", 128)
        self.uiRamSpinBox.setObjectName(_fromUtf8("uiRamSpinBox"))
        self.gridLayout_5.addWidget(self.uiRamSpinBox, 0, 1, 1, 1)
        self.uiNvramLabel = QtGui.QLabel(self.uiMemoriesGroupBox)
        self.uiNvramLabel.setObjectName(_fromUtf8("uiNvramLabel"))
        self.gridLayout_5.addWidget(self.uiNvramLabel, 1, 0, 1, 1)
        self.uiNvramSpinBox = QtGui.QSpinBox(self.uiMemoriesGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNvramSpinBox.sizePolicy().hasHeightForWidth())
        self.uiNvramSpinBox.setSizePolicy(sizePolicy)
        self.uiNvramSpinBox.setMinimum(32)
        self.uiNvramSpinBox.setMaximum(65535)
        self.uiNvramSpinBox.setSingleStep(32)
        self.uiNvramSpinBox.setProperty("value", 128)
        self.uiNvramSpinBox.setObjectName(_fromUtf8("uiNvramSpinBox"))
        self.gridLayout_5.addWidget(self.uiNvramSpinBox, 1, 1, 1, 1)
        self.uiIomemLabel = QtGui.QLabel(self.uiMemoriesGroupBox)
        self.uiIomemLabel.setObjectName(_fromUtf8("uiIomemLabel"))
        self.gridLayout_5.addWidget(self.uiIomemLabel, 2, 0, 1, 1)
        self.uiIomemSpinBox = QtGui.QSpinBox(self.uiMemoriesGroupBox)
        self.uiIomemSpinBox.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIomemSpinBox.sizePolicy().hasHeightForWidth())
        self.uiIomemSpinBox.setSizePolicy(sizePolicy)
        self.uiIomemSpinBox.setMaximum(100)
        self.uiIomemSpinBox.setSingleStep(5)
        self.uiIomemSpinBox.setProperty("value", 5)
        self.uiIomemSpinBox.setObjectName(_fromUtf8("uiIomemSpinBox"))
        self.gridLayout_5.addWidget(self.uiIomemSpinBox, 2, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.uiMemoriesGroupBox)
        self.uiDisksGroupBox = QtGui.QGroupBox(self.uiMemoriesPageWidget)
        self.uiDisksGroupBox.setObjectName(_fromUtf8("uiDisksGroupBox"))
        self.gridlayout = QtGui.QGridLayout(self.uiDisksGroupBox)
        self.gridlayout.setObjectName(_fromUtf8("gridlayout"))
        self.uiDisk0Label = QtGui.QLabel(self.uiDisksGroupBox)
        self.uiDisk0Label.setObjectName(_fromUtf8("uiDisk0Label"))
        self.gridlayout.addWidget(self.uiDisk0Label, 0, 0, 1, 1)
        self.uiDisk0SpinBox = QtGui.QSpinBox(self.uiDisksGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk0SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk0SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk0SpinBox.setMaximum(99999)
        self.uiDisk0SpinBox.setSingleStep(4)
        self.uiDisk0SpinBox.setObjectName(_fromUtf8("uiDisk0SpinBox"))
        self.gridlayout.addWidget(self.uiDisk0SpinBox, 0, 1, 1, 1)
        self.uiDisk1Label = QtGui.QLabel(self.uiDisksGroupBox)
        self.uiDisk1Label.setObjectName(_fromUtf8("uiDisk1Label"))
        self.gridlayout.addWidget(self.uiDisk1Label, 1, 0, 1, 1)
        self.uiDisk1SpinBox = QtGui.QSpinBox(self.uiDisksGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk1SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk1SpinBox.setMaximum(99999)
        self.uiDisk1SpinBox.setSingleStep(4)
        self.uiDisk1SpinBox.setObjectName(_fromUtf8("uiDisk1SpinBox"))
        self.gridlayout.addWidget(self.uiDisk1SpinBox, 1, 1, 1, 1)
        self.verticalLayout_2.addWidget(self.uiDisksGroupBox)
        spacerItem1 = QtGui.QSpacerItem(20, 21, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.uiTabWidget.addTab(self.uiMemoriesPageWidget, _fromUtf8(""))
        self.uiSlotsPageWidget = QtGui.QWidget()
        self.uiSlotsPageWidget.setObjectName(_fromUtf8("uiSlotsPageWidget"))
        self.verticalLayout = QtGui.QVBoxLayout(self.uiSlotsPageWidget)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.uiAdaptersGroupBox = QtGui.QGroupBox(self.uiSlotsPageWidget)
        self.uiAdaptersGroupBox.setObjectName(_fromUtf8("uiAdaptersGroupBox"))
        self.gridLayout = QtGui.QGridLayout(self.uiAdaptersGroupBox)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiSlot1comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot1comboBox.setObjectName(_fromUtf8("uiSlot1comboBox"))
        self.gridLayout.addWidget(self.uiSlot1comboBox, 1, 1, 1, 1)
        self.uiSlot0Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot0Label.sizePolicy().hasHeightForWidth())
        self.uiSlot0Label.setSizePolicy(sizePolicy)
        self.uiSlot0Label.setObjectName(_fromUtf8("uiSlot0Label"))
        self.gridLayout.addWidget(self.uiSlot0Label, 0, 0, 1, 1)
        self.uiSlot0comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot0comboBox.setObjectName(_fromUtf8("uiSlot0comboBox"))
        self.gridLayout.addWidget(self.uiSlot0comboBox, 0, 1, 1, 1)
        self.uiSlot1Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot1Label.sizePolicy().hasHeightForWidth())
        self.uiSlot1Label.setSizePolicy(sizePolicy)
        self.uiSlot1Label.setObjectName(_fromUtf8("uiSlot1Label"))
        self.gridLayout.addWidget(self.uiSlot1Label, 1, 0, 1, 1)
        self.uiSlot2Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot2Label.sizePolicy().hasHeightForWidth())
        self.uiSlot2Label.setSizePolicy(sizePolicy)
        self.uiSlot2Label.setObjectName(_fromUtf8("uiSlot2Label"))
        self.gridLayout.addWidget(self.uiSlot2Label, 2, 0, 1, 1)
        self.uiSlot2comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot2comboBox.setObjectName(_fromUtf8("uiSlot2comboBox"))
        self.gridLayout.addWidget(self.uiSlot2comboBox, 2, 1, 1, 1)
        self.uiSlot3Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot3Label.sizePolicy().hasHeightForWidth())
        self.uiSlot3Label.setSizePolicy(sizePolicy)
        self.uiSlot3Label.setObjectName(_fromUtf8("uiSlot3Label"))
        self.gridLayout.addWidget(self.uiSlot3Label, 3, 0, 1, 1)
        self.uiSlot3comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot3comboBox.setObjectName(_fromUtf8("uiSlot3comboBox"))
        self.gridLayout.addWidget(self.uiSlot3comboBox, 3, 1, 1, 1)
        self.uiSlot4Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot4Label.sizePolicy().hasHeightForWidth())
        self.uiSlot4Label.setSizePolicy(sizePolicy)
        self.uiSlot4Label.setObjectName(_fromUtf8("uiSlot4Label"))
        self.gridLayout.addWidget(self.uiSlot4Label, 4, 0, 1, 1)
        self.uiSlot4comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot4comboBox.setObjectName(_fromUtf8("uiSlot4comboBox"))
        self.gridLayout.addWidget(self.uiSlot4comboBox, 4, 1, 1, 1)
        self.uiSlot5Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot5Label.sizePolicy().hasHeightForWidth())
        self.uiSlot5Label.setSizePolicy(sizePolicy)
        self.uiSlot5Label.setObjectName(_fromUtf8("uiSlot5Label"))
        self.gridLayout.addWidget(self.uiSlot5Label, 5, 0, 1, 1)
        self.uiSlot5comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot5comboBox.setObjectName(_fromUtf8("uiSlot5comboBox"))
        self.gridLayout.addWidget(self.uiSlot5comboBox, 5, 1, 1, 1)
        self.uiSlot6Label = QtGui.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot6Label.sizePolicy().hasHeightForWidth())
        self.uiSlot6Label.setSizePolicy(sizePolicy)
        self.uiSlot6Label.setObjectName(_fromUtf8("uiSlot6Label"))
        self.gridLayout.addWidget(self.uiSlot6Label, 6, 0, 1, 1)
        self.uiSlot6comboBox = QtGui.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot6comboBox.setObjectName(_fromUtf8("uiSlot6comboBox"))
        self.gridLayout.addWidget(self.uiSlot6comboBox, 6, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiAdaptersGroupBox)
        self.uiWicsGroupBox = QtGui.QGroupBox(self.uiSlotsPageWidget)
        self.uiWicsGroupBox.setObjectName(_fromUtf8("uiWicsGroupBox"))
        self.gridlayout1 = QtGui.QGridLayout(self.uiWicsGroupBox)
        self.gridlayout1.setObjectName(_fromUtf8("gridlayout1"))
        self.uiWic0Label = QtGui.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic0Label.sizePolicy().hasHeightForWidth())
        self.uiWic0Label.setSizePolicy(sizePolicy)
        self.uiWic0Label.setObjectName(_fromUtf8("uiWic0Label"))
        self.gridlayout1.addWidget(self.uiWic0Label, 0, 0, 1, 1)
        self.uiWic0comboBox = QtGui.QComboBox(self.uiWicsGroupBox)
        self.uiWic0comboBox.setObjectName(_fromUtf8("uiWic0comboBox"))
        self.gridlayout1.addWidget(self.uiWic0comboBox, 0, 1, 1, 1)
        self.uiWic1Label = QtGui.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic1Label.sizePolicy().hasHeightForWidth())
        self.uiWic1Label.setSizePolicy(sizePolicy)
        self.uiWic1Label.setObjectName(_fromUtf8("uiWic1Label"))
        self.gridlayout1.addWidget(self.uiWic1Label, 1, 0, 1, 1)
        self.uiWic1comboBox = QtGui.QComboBox(self.uiWicsGroupBox)
        self.uiWic1comboBox.setObjectName(_fromUtf8("uiWic1comboBox"))
        self.gridlayout1.addWidget(self.uiWic1comboBox, 1, 1, 1, 1)
        self.uiWic2Label = QtGui.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic2Label.sizePolicy().hasHeightForWidth())
        self.uiWic2Label.setSizePolicy(sizePolicy)
        self.uiWic2Label.setObjectName(_fromUtf8("uiWic2Label"))
        self.gridlayout1.addWidget(self.uiWic2Label, 2, 0, 1, 1)
        self.uiWic2comboBox = QtGui.QComboBox(self.uiWicsGroupBox)
        self.uiWic2comboBox.setObjectName(_fromUtf8("uiWic2comboBox"))
        self.gridlayout1.addWidget(self.uiWic2comboBox, 2, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiWicsGroupBox)
        spacerItem2 = QtGui.QSpacerItem(325, 31, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.uiTabWidget.addTab(self.uiSlotsPageWidget, _fromUtf8(""))
        self.uiAdvancedPageWidget = QtGui.QWidget()
        self.uiAdvancedPageWidget.setObjectName(_fromUtf8("uiAdvancedPageWidget"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.uiAdvancedPageWidget)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.uiSystemGroupBox = QtGui.QGroupBox(self.uiAdvancedPageWidget)
        self.uiSystemGroupBox.setObjectName(_fromUtf8("uiSystemGroupBox"))
        self.gridLayout_6 = QtGui.QGridLayout(self.uiSystemGroupBox)
        self.gridLayout_6.setObjectName(_fromUtf8("gridLayout_6"))
        self.uiSystemIdLineEdit = QtGui.QLineEdit(self.uiSystemGroupBox)
        self.uiSystemIdLineEdit.setObjectName(_fromUtf8("uiSystemIdLineEdit"))
        self.gridLayout_6.addWidget(self.uiSystemIdLineEdit, 0, 1, 1, 1)
        self.label = QtGui.QLabel(self.uiSystemGroupBox)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout_6.addWidget(self.label, 0, 0, 1, 1)
        self.uiBaseMacLabel = QtGui.QLabel(self.uiSystemGroupBox)
        self.uiBaseMacLabel.setObjectName(_fromUtf8("uiBaseMacLabel"))
        self.gridLayout_6.addWidget(self.uiBaseMacLabel, 1, 0, 1, 1)
        self.uiBaseMACLineEdit = QtGui.QLineEdit(self.uiSystemGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiBaseMACLineEdit.sizePolicy().hasHeightForWidth())
        self.uiBaseMACLineEdit.setSizePolicy(sizePolicy)
        self.uiBaseMACLineEdit.setText(_fromUtf8(""))
        self.uiBaseMACLineEdit.setObjectName(_fromUtf8("uiBaseMACLineEdit"))
        self.gridLayout_6.addWidget(self.uiBaseMACLineEdit, 1, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiSystemGroupBox)
        self.uiOptimizationsGroupBox = QtGui.QGroupBox(self.uiAdvancedPageWidget)
        self.uiOptimizationsGroupBox.setObjectName(_fromUtf8("uiOptimizationsGroupBox"))
        self.gridLayout_4 = QtGui.QGridLayout(self.uiOptimizationsGroupBox)
        self.gridLayout_4.setObjectName(_fromUtf8("gridLayout_4"))
        self.uiExecAreaLabel = QtGui.QLabel(self.uiOptimizationsGroupBox)
        self.uiExecAreaLabel.setObjectName(_fromUtf8("uiExecAreaLabel"))
        self.gridLayout_4.addWidget(self.uiExecAreaLabel, 3, 0, 1, 1)
        self.uiExecAreaSpinBox = QtGui.QSpinBox(self.uiOptimizationsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiExecAreaSpinBox.sizePolicy().hasHeightForWidth())
        self.uiExecAreaSpinBox.setSizePolicy(sizePolicy)
        self.uiExecAreaSpinBox.setMaximum(4096)
        self.uiExecAreaSpinBox.setSingleStep(4)
        self.uiExecAreaSpinBox.setProperty("value", 64)
        self.uiExecAreaSpinBox.setObjectName(_fromUtf8("uiExecAreaSpinBox"))
        self.gridLayout_4.addWidget(self.uiExecAreaSpinBox, 3, 1, 1, 1)
        self.uiSparseMemoryCheckBox = QtGui.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiSparseMemoryCheckBox.setChecked(False)
        self.uiSparseMemoryCheckBox.setObjectName(_fromUtf8("uiSparseMemoryCheckBox"))
        self.gridLayout_4.addWidget(self.uiSparseMemoryCheckBox, 5, 0, 1, 2)
        self.uiIdlepcLabel = QtGui.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlepcLabel.setObjectName(_fromUtf8("uiIdlepcLabel"))
        self.gridLayout_4.addWidget(self.uiIdlepcLabel, 0, 0, 1, 1)
        self.uiIdlepcLineEdit = QtGui.QLineEdit(self.uiOptimizationsGroupBox)
        self.uiIdlepcLineEdit.setObjectName(_fromUtf8("uiIdlepcLineEdit"))
        self.gridLayout_4.addWidget(self.uiIdlepcLineEdit, 0, 1, 1, 1)
        self.uiIdlemaxLabel = QtGui.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlemaxLabel.setObjectName(_fromUtf8("uiIdlemaxLabel"))
        self.gridLayout_4.addWidget(self.uiIdlemaxLabel, 1, 0, 1, 1)
        self.uiIdlemaxSpinBox = QtGui.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlemaxSpinBox.setMinimum(100)
        self.uiIdlemaxSpinBox.setMaximum(10000)
        self.uiIdlemaxSpinBox.setSingleStep(100)
        self.uiIdlemaxSpinBox.setProperty("value", 500)
        self.uiIdlemaxSpinBox.setObjectName(_fromUtf8("uiIdlemaxSpinBox"))
        self.gridLayout_4.addWidget(self.uiIdlemaxSpinBox, 1, 1, 1, 1)
        self.uiIdlesleepLabel = QtGui.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlesleepLabel.setObjectName(_fromUtf8("uiIdlesleepLabel"))
        self.gridLayout_4.addWidget(self.uiIdlesleepLabel, 2, 0, 1, 1)
        self.uiMmapCheckBox = QtGui.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiMmapCheckBox.setChecked(True)
        self.uiMmapCheckBox.setObjectName(_fromUtf8("uiMmapCheckBox"))
        self.gridLayout_4.addWidget(self.uiMmapCheckBox, 4, 0, 1, 2)
        self.uiIdlesleepSpinBox = QtGui.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlesleepSpinBox.setMinimum(1)
        self.uiIdlesleepSpinBox.setMaximum(500)
        self.uiIdlesleepSpinBox.setSingleStep(10)
        self.uiIdlesleepSpinBox.setProperty("value", 30)
        self.uiIdlesleepSpinBox.setObjectName(_fromUtf8("uiIdlesleepSpinBox"))
        self.gridLayout_4.addWidget(self.uiIdlesleepSpinBox, 2, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiOptimizationsGroupBox)
        spacerItem3 = QtGui.QSpacerItem(304, 251, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem3)
        self.uiTabWidget.addTab(self.uiAdvancedPageWidget, _fromUtf8(""))
        self.uiEnvironmentPageWidget = QtGui.QWidget()
        self.uiEnvironmentPageWidget.setObjectName(_fromUtf8("uiEnvironmentPageWidget"))
        self.verticalLayout_3 = QtGui.QVBoxLayout(self.uiEnvironmentPageWidget)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.uiPowerSuppliesGroupBox = QtGui.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiPowerSuppliesGroupBox.setObjectName(_fromUtf8("uiPowerSuppliesGroupBox"))
        self.gridLayout_7 = QtGui.QGridLayout(self.uiPowerSuppliesGroupBox)
        self.gridLayout_7.setObjectName(_fromUtf8("gridLayout_7"))
        self.uiPowerSupply2Label = QtGui.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply2Label.setObjectName(_fromUtf8("uiPowerSupply2Label"))
        self.gridLayout_7.addWidget(self.uiPowerSupply2Label, 1, 0, 1, 1)
        self.uiPowerSupply1ComboBox = QtGui.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply1ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply1ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply1ComboBox.setObjectName(_fromUtf8("uiPowerSupply1ComboBox"))
        self.uiPowerSupply1ComboBox.addItem(_fromUtf8(""))
        self.uiPowerSupply1ComboBox.addItem(_fromUtf8(""))
        self.gridLayout_7.addWidget(self.uiPowerSupply1ComboBox, 0, 1, 1, 1)
        self.uiPowerSupply1Label = QtGui.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply1Label.setObjectName(_fromUtf8("uiPowerSupply1Label"))
        self.gridLayout_7.addWidget(self.uiPowerSupply1Label, 0, 0, 1, 1)
        self.uiPowerSupply2ComboBox = QtGui.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply2ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply2ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply2ComboBox.setObjectName(_fromUtf8("uiPowerSupply2ComboBox"))
        self.uiPowerSupply2ComboBox.addItem(_fromUtf8(""))
        self.uiPowerSupply2ComboBox.addItem(_fromUtf8(""))
        self.gridLayout_7.addWidget(self.uiPowerSupply2ComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiPowerSuppliesGroupBox)
        self.uiSensorsGroupBox = QtGui.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiSensorsGroupBox.setObjectName(_fromUtf8("uiSensorsGroupBox"))
        self.gridLayout_3 = QtGui.QGridLayout(self.uiSensorsGroupBox)
        self.gridLayout_3.setObjectName(_fromUtf8("gridLayout_3"))
        self.uiSensor1Label = QtGui.QLabel(self.uiSensorsGroupBox)
        self.uiSensor1Label.setObjectName(_fromUtf8("uiSensor1Label"))
        self.gridLayout_3.addWidget(self.uiSensor1Label, 0, 0, 1, 1)
        self.uiSensor1SpinBox = QtGui.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor1SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor1SpinBox.setMaximum(100)
        self.uiSensor1SpinBox.setObjectName(_fromUtf8("uiSensor1SpinBox"))
        self.gridLayout_3.addWidget(self.uiSensor1SpinBox, 0, 1, 1, 1)
        self.uiSensor2Label = QtGui.QLabel(self.uiSensorsGroupBox)
        self.uiSensor2Label.setObjectName(_fromUtf8("uiSensor2Label"))
        self.gridLayout_3.addWidget(self.uiSensor2Label, 1, 0, 1, 1)
        self.uiSensor2SpinBox = QtGui.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor2SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor2SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor2SpinBox.setMaximum(100)
        self.uiSensor2SpinBox.setObjectName(_fromUtf8("uiSensor2SpinBox"))
        self.gridLayout_3.addWidget(self.uiSensor2SpinBox, 1, 1, 1, 1)
        self.uiSensor3Label = QtGui.QLabel(self.uiSensorsGroupBox)
        self.uiSensor3Label.setObjectName(_fromUtf8("uiSensor3Label"))
        self.gridLayout_3.addWidget(self.uiSensor3Label, 2, 0, 1, 1)
        self.uiSensor3SpinBox = QtGui.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor3SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor3SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor3SpinBox.setMaximum(100)
        self.uiSensor3SpinBox.setObjectName(_fromUtf8("uiSensor3SpinBox"))
        self.gridLayout_3.addWidget(self.uiSensor3SpinBox, 2, 1, 1, 1)
        self.uiSensor4Label = QtGui.QLabel(self.uiSensorsGroupBox)
        self.uiSensor4Label.setObjectName(_fromUtf8("uiSensor4Label"))
        self.gridLayout_3.addWidget(self.uiSensor4Label, 3, 0, 1, 1)
        self.uiSensor4SpinBox = QtGui.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor4SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor4SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor4SpinBox.setMaximum(100)
        self.uiSensor4SpinBox.setObjectName(_fromUtf8("uiSensor4SpinBox"))
        self.gridLayout_3.addWidget(self.uiSensor4SpinBox, 3, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiSensorsGroupBox)
        spacerItem4 = QtGui.QSpacerItem(20, 194, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem4)
        self.uiTabWidget.addTab(self.uiEnvironmentPageWidget, _fromUtf8(""))
        self.vboxlayout.addWidget(self.uiTabWidget)

        self.retranslateUi(iosRouterConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        self.uiPowerSupply1ComboBox.setCurrentIndex(1)
        self.uiPowerSupply2ComboBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(iosRouterConfigPageWidget)
        iosRouterConfigPageWidget.setTabOrder(self.uiMidplaneComboBox, self.uiNPEComboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiNPEComboBox, self.uiRamSpinBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiRamSpinBox, self.uiNvramSpinBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiNvramSpinBox, self.uiDisk0SpinBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiDisk0SpinBox, self.uiDisk1SpinBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiDisk1SpinBox, self.uiSlot0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot0comboBox, self.uiSlot1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot1comboBox, self.uiSlot2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot2comboBox, self.uiSlot3comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot3comboBox, self.uiSlot4comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot4comboBox, self.uiSlot5comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot5comboBox, self.uiSlot6comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot6comboBox, self.uiWic0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic0comboBox, self.uiWic1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic1comboBox, self.uiWic2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic2comboBox, self.uiBaseMACLineEdit)
        iosRouterConfigPageWidget.setTabOrder(self.uiBaseMACLineEdit, self.uiExecAreaSpinBox)

    def retranslateUi(self, iosRouterConfigPageWidget):
        iosRouterConfigPageWidget.setWindowTitle(_translate("iosRouterConfigPageWidget", "Dynamips IOS Router configuration", None))
        self.uiNameLabel.setText(_translate("iosRouterConfigPageWidget", "Name:", None))
        self.uiPlatformLabel.setText(_translate("iosRouterConfigPageWidget", "Platform:", None))
        self.uiChassisLabel.setText(_translate("iosRouterConfigPageWidget", "Chassis:", None))
        self.uiIOSImageLabel.setText(_translate("iosRouterConfigPageWidget", "IOS image path:", None))
        self.uiIOSImageToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse...", None))
        self.uiStartupConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial startup-config:", None))
        self.uiStartupConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse...", None))
        self.uiPrivateConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial private-config:", None))
        self.uiPrivateConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse...", None))
        self.uiConsolePortLabel.setText(_translate("iosRouterConfigPageWidget", "Console port:", None))
        self.uiAuxPortLabel.setText(_translate("iosRouterConfigPageWidget", "Aux port:", None))
        self.uiMidplaneLabel.setText(_translate("iosRouterConfigPageWidget", "Midplane:", None))
        self.uiNPELabel.setText(_translate("iosRouterConfigPageWidget", "NPE:", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralPageWidget), _translate("iosRouterConfigPageWidget", "General", None))
        self.uiMemoriesGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Memories", None))
        self.uiRamLabel.setText(_translate("iosRouterConfigPageWidget", "RAM size:", None))
        self.uiRamSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB", None))
        self.uiNvramLabel.setText(_translate("iosRouterConfigPageWidget", "NVRAM size:", None))
        self.uiNvramSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " KiB", None))
        self.uiIomemLabel.setText(_translate("iosRouterConfigPageWidget", "I/O memory :", None))
        self.uiIomemSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " %", None))
        self.uiDisksGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Disks", None))
        self.uiDisk0Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk0 size:", None))
        self.uiDisk0SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB", None))
        self.uiDisk1Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk1 size:", None))
        self.uiDisk1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiMemoriesPageWidget), _translate("iosRouterConfigPageWidget", "Memories and disks", None))
        self.uiAdaptersGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Adapters", None))
        self.uiSlot0Label.setText(_translate("iosRouterConfigPageWidget", "slot 0:", None))
        self.uiSlot1Label.setText(_translate("iosRouterConfigPageWidget", "slot 1:", None))
        self.uiSlot2Label.setText(_translate("iosRouterConfigPageWidget", "slot 2:", None))
        self.uiSlot3Label.setText(_translate("iosRouterConfigPageWidget", "slot 3:", None))
        self.uiSlot4Label.setText(_translate("iosRouterConfigPageWidget", "slot 4:", None))
        self.uiSlot5Label.setText(_translate("iosRouterConfigPageWidget", "slot 5:", None))
        self.uiSlot6Label.setText(_translate("iosRouterConfigPageWidget", "slot 6:", None))
        self.uiWicsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "WICs", None))
        self.uiWic0Label.setText(_translate("iosRouterConfigPageWidget", "wic 0:", None))
        self.uiWic1Label.setText(_translate("iosRouterConfigPageWidget", "wic 1:", None))
        self.uiWic2Label.setText(_translate("iosRouterConfigPageWidget", "wic 2:", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiSlotsPageWidget), _translate("iosRouterConfigPageWidget", "Slots", None))
        self.uiSystemGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "System", None))
        self.label.setText(_translate("iosRouterConfigPageWidget", "System ID:", None))
        self.uiBaseMacLabel.setText(_translate("iosRouterConfigPageWidget", "Base MAC:", None))
        self.uiOptimizationsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Optimisations", None))
        self.uiExecAreaLabel.setText(_translate("iosRouterConfigPageWidget", "Exec area:", None))
        self.uiExecAreaSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB", None))
        self.uiSparseMemoryCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable sparse memory support", None))
        self.uiIdlepcLabel.setText(_translate("iosRouterConfigPageWidget", "Idle-PC:", None))
        self.uiIdlemaxLabel.setText(_translate("iosRouterConfigPageWidget", "Idlemax:", None))
        self.uiIdlesleepLabel.setText(_translate("iosRouterConfigPageWidget", "Idlesleep:", None))
        self.uiMmapCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable mmap support", None))
        self.uiIdlesleepSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " ms", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiAdvancedPageWidget), _translate("iosRouterConfigPageWidget", "Advanced", None))
        self.uiPowerSuppliesGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Power supplies", None))
        self.uiPowerSupply2Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 2:", None))
        self.uiPowerSupply1ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on", None))
        self.uiPowerSupply1ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off", None))
        self.uiPowerSupply1Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 1:", None))
        self.uiPowerSupply2ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on", None))
        self.uiPowerSupply2ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off", None))
        self.uiSensorsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Temperature sensors", None))
        self.uiSensor1Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller inlet:", None))
        self.uiSensor1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C", None))
        self.uiSensor2Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller outlet:", None))
        self.uiSensor2SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C", None))
        self.uiSensor3Label.setText(_translate("iosRouterConfigPageWidget", "NPE inlet:", None))
        self.uiSensor3SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C", None))
        self.uiSensor4Label.setText(_translate("iosRouterConfigPageWidget", "NPE outlet:", None))
        self.uiSensor4SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C", None))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiEnvironmentPageWidget), _translate("iosRouterConfigPageWidget", "Environment", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/iou/ui/iou_device_preferences_page.ui'
#
# Created: Wed Mar 11 22:01:55 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_IOUDevicePreferencesPageWidget(object):
    def setupUi(self, IOUDevicePreferencesPageWidget):
        IOUDevicePreferencesPageWidget.setObjectName(_fromUtf8("IOUDevicePreferencesPageWidget"))
        IOUDevicePreferencesPageWidget.resize(506, 508)
        self.gridLayout = QtGui.QGridLayout(IOUDevicePreferencesPageWidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.uiIOUDevicesTreeWidget = QtGui.QTreeWidget(IOUDevicePreferencesPageWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIOUDevicesTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiIOUDevicesTreeWidget.setSizePolicy(sizePolicy)
        self.uiIOUDevicesTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiIOUDevicesTreeWidget.setFont(font)
        self.uiIOUDevicesTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiIOUDevicesTreeWidget.setRootIsDecorated(False)
        self.uiIOUDevicesTreeWidget.setObjectName(_fromUtf8("uiIOUDevicesTreeWidget"))
        self.uiIOUDevicesTreeWidget.headerItem().setText(0, _fromUtf8("1"))
        self.uiIOUDevicesTreeWidget.header().setVisible(False)
        self.gridLayout.addWidget(self.uiIOUDevicesTreeWidget, 0, 0, 2, 1)
        self.uiIOUDeviceInfoTreeWidget = QtGui.QTreeWidget(IOUDevicePreferencesPageWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIOUDeviceInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiIOUDeviceInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiIOUDeviceInfoTreeWidget.setIndentation(10)
        self.uiIOUDeviceInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiIOUDeviceInfoTreeWidget.setObjectName(_fromUtf8("uiIOUDeviceInfoTreeWidget"))
        self.uiIOUDeviceInfoTreeWidget.header().setVisible(False)
        self.gridLayout.addWidget(self.uiIOUDeviceInfoTreeWidget, 0, 1, 1, 1)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.uiNewIOUDevicePushButton = QtGui.QPushButton(IOUDevicePreferencesPageWidget)
        self.uiNewIOUDevicePushButton.setObjectName(_fromUtf8("uiNewIOUDevicePushButton"))
        self.horizontalLayout_6.addWidget(self.uiNewIOUDevicePushButton)
        self.uiEditIOUDevicePushButton = QtGui.QPushButton(IOUDevicePreferencesPageWidget)
        self.uiEditIOUDevicePushButton.setEnabled(False)
        self.uiEditIOUDevicePushButton.setObjectName(_fromUtf8("uiEditIOUDevicePushButton"))
        self.horizontalLayout_6.addWidget(self.uiEditIOUDevicePushButton)
        self.uiDeleteIOUDevicePushButton = QtGui.QPushButton(IOUDevicePreferencesPageWidget)
        self.uiDeleteIOUDevicePushButton.setEnabled(False)
        self.uiDeleteIOUDevicePushButton.setObjectName(_fromUtf8("uiDeleteIOUDevicePushButton"))
        self.horizontalLayout_6.addWidget(self.uiDeleteIOUDevicePushButton)
        self.gridLayout.addLayout(self.horizontalLayout_6, 1, 1, 1, 1)

        self.retranslateUi(IOUDevicePreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(IOUDevicePreferencesPageWidget)

    def retranslateUi(self, IOUDevicePreferencesPageWidget):
        IOUDevicePreferencesPageWidget.setWindowTitle(_translate("IOUDevicePreferencesPageWidget", "IOU devices", None))
        IOUDevicePreferencesPageWidget.setAccessibleName(_translate("IOUDevicePreferencesPageWidget", "IOU device templates", None))
        self.uiIOUDeviceInfoTreeWidget.headerItem().setText(0, _translate("IOUDevicePreferencesPageWidget", "1", None))
        self.uiIOUDeviceInfoTreeWidget.headerItem().setText(1, _translate("IOUDevicePreferencesPageWidget", "2", None))
        self.uiNewIOUDevicePushButton.setText(_translate("IOUDevicePreferencesPageWidget", "&New", None))
        self.uiEditIOUDevicePushButton.setText(_translate("IOUDevicePreferencesPageWidget", "&Edit", None))
        self.uiDeleteIOUDevicePushButton.setText(_translate("IOUDevicePreferencesPageWidget", "&Delete", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/node_configurator_dialog.ui'
#
# Created: Sun Aug 17 18:05:14 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_NodeConfiguratorDialog(object):

    def setupUi(self, NodeConfiguratorDialog):
        NodeConfiguratorDialog.setObjectName(_fromUtf8("NodeConfiguratorDialog"))
        NodeConfiguratorDialog.resize(689, 454)
        self.gridlayout = QtGui.QGridLayout(NodeConfiguratorDialog)
        self.gridlayout.setObjectName(_fromUtf8("gridlayout"))
        self.splitter = QtGui.QSplitter(NodeConfiguratorDialog)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName(_fromUtf8("splitter"))
        self.uiNodesTreeWidget = QtGui.QTreeWidget(self.splitter)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNodesTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiNodesTreeWidget.setSizePolicy(sizePolicy)
        self.uiNodesTreeWidget.setObjectName(_fromUtf8("uiNodesTreeWidget"))
        self.uiNodesTreeWidget.header().setVisible(False)
        self.verticalLayout = QtGui.QWidget(self.splitter)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.vboxlayout = QtGui.QVBoxLayout(self.verticalLayout)
        self.vboxlayout.setSpacing(4)
        self.vboxlayout.setMargin(0)
        self.vboxlayout.setObjectName(_fromUtf8("vboxlayout"))
        self.uiTitleLabel = QtGui.QLabel(self.verticalLayout)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.uiTitleLabel.setFont(font)
        self.uiTitleLabel.setFrameShape(QtGui.QFrame.Box)
        self.uiTitleLabel.setFrameShadow(QtGui.QFrame.Sunken)
        self.uiTitleLabel.setTextFormat(QtCore.Qt.PlainText)
        self.uiTitleLabel.setObjectName(_fromUtf8("uiTitleLabel"))
        self.vboxlayout.addWidget(self.uiTitleLabel)
        self.uiConfigStackedWidget = QtGui.QStackedWidget(self.verticalLayout)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConfigStackedWidget.sizePolicy().hasHeightForWidth())
        self.uiConfigStackedWidget.setSizePolicy(sizePolicy)
        self.uiConfigStackedWidget.setFrameShape(QtGui.QFrame.Box)
        self.uiConfigStackedWidget.setFrameShadow(QtGui.QFrame.Sunken)
        self.uiConfigStackedWidget.setObjectName(_fromUtf8("uiConfigStackedWidget"))
        self.uiEmptyPageWidget = QtGui.QWidget()
        self.uiEmptyPageWidget.setObjectName(_fromUtf8("uiEmptyPageWidget"))
        self.vboxlayout1 = QtGui.QVBoxLayout(self.uiEmptyPageWidget)
        self.vboxlayout1.setSpacing(0)
        self.vboxlayout1.setContentsMargins(0, 4, 0, 0)
        self.vboxlayout1.setObjectName(_fromUtf8("vboxlayout1"))
        spacerItem = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.vboxlayout1.addItem(spacerItem)
        self.textLabel = QtGui.QLabel(self.uiEmptyPageWidget)
        self.textLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.textLabel.setObjectName(_fromUtf8("textLabel"))
        self.vboxlayout1.addWidget(self.textLabel)
        spacerItem1 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.vboxlayout1.addItem(spacerItem1)
        self.uiConfigStackedWidget.addWidget(self.uiEmptyPageWidget)
        self.vboxlayout.addWidget(self.uiConfigStackedWidget)
        self.gridlayout.addWidget(self.splitter, 0, 0, 1, 1)
        self.uiButtonBox = QtGui.QDialogButtonBox(NodeConfiguratorDialog)
        self.uiButtonBox.setOrientation(QtCore.Qt.Horizontal)
        self.uiButtonBox.setStandardButtons(QtGui.QDialogButtonBox.Apply | QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Reset)
        self.uiButtonBox.setObjectName(_fromUtf8("uiButtonBox"))
        self.gridlayout.addWidget(self.uiButtonBox, 1, 0, 1, 1)

        self.retranslateUi(NodeConfiguratorDialog)
        self.uiConfigStackedWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(NodeConfiguratorDialog)

    def retranslateUi(self, NodeConfiguratorDialog):
        NodeConfiguratorDialog.setWindowTitle(_translate("NodeConfiguratorDialog", "Node configurator", None))
        self.uiNodesTreeWidget.headerItem().setText(0, _translate("NodeConfiguratorDialog", "Nodes", None))
        self.uiTitleLabel.setText(_translate("NodeConfiguratorDialog", "Node Configuration", None))
        self.textLabel.setText(_translate("NodeConfiguratorDialog", "Please select a node in the list \n"
                                          "to display the configuration page.", None))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Interacts with Rackspace API to create and manage cloud instances. """

from .base_cloud_ctrl import BaseCloudCtrl
import json
import requests
from libcloud.compute.drivers.rackspace import ENDPOINT_ARGS_MAP
from libcloud.compute.providers import get_driver
from libcloud.compute.types import Provider
from libcloud.storage.providers import get_driver as get_storage_driver
from libcloud.storage.types import Provider as StorageProvider

from .exceptions import ItemNotFound, ApiError
from ..version import __version__

from collections import OrderedDict

import logging
log = logging.getLogger(__name__)

RACKSPACE_REGIONS = [{ENDPOINT_ARGS_MAP[k]['region']: k} for k in
                     ENDPOINT_ARGS_MAP]


class RackspaceCtrl(BaseCloudCtrl):

    """ Controller class for interacting with Rackspace API. """

    def __init__(self, username, api_key, *args, **kwargs):
        super(RackspaceCtrl, self).__init__(username, api_key)

        # set this up so it can be swapped out with a mock for testing
        self.post_fn = requests.post
        self.driver_cls = get_driver(Provider.RACKSPACE)
        self.storage_driver_cls = get_storage_driver(StorageProvider.CLOUDFILES)

        self.driver = None
        self.storage_driver = None
        self.region = None
        self.instances = {}

        self.authenticated = False
        self.identity_ep = \
            "https://identity.api.rackspacecloud.com/v2.0/tokens"

        self.regions = []
        self.token = None
        self.tenant_id = None
        self.flavor_ep = "https://dfw.servers.api.rackspacecloud.com/v2/{username}/flavors"
        self._flavors = OrderedDict([
            ('2', '512MB, 1 VCPU'),
            ('3', '1GB, 1 VCPU'),
            ('4', '2GB, 2 VCPUs'),
            ('5', '4GB, 2 VCPUs'),
            ('6', '8GB, 4 VCPUs'),
            ('7', '15GB, 6 VCPUs'),
            ('8', '30GB, 8 VCPUs'),
            ('performance1-1', '1GB Performance, 1 VCPU'),
            ('performance1-2', '2GB Performance, 2 VCPUs'),
            ('performance1-4', '4GB Performance, 4 VCPUs'),
            ('performance1-8', '8GB Performance, 8 VCPUs'),
            ('performance2-15', '15GB Performance, 4 VCPUs'),
            ('performance2-30', '30GB Performance, 8 VCPUs'),
            ('performance2-60', '60GB Performance, 16 VCPUs'),
            ('performance2-90', '90GB Performance, 24 VCPUs'),
            ('performance2-120', '120GB Performance, 32 VCPUs',)
        ])

    def authenticate(self):
        """
        Submit username and api key to API service.

        If authentication is successful, set self.regions and self.token.
        Return boolean.

        """

        self.authenticated = False

        if len(self.username) < 1:
            return False

        if len(self.api_key) < 1:
            return False

        data = json.dumps({
            "auth": {
                "RAX-KSKEY:apiKeyCredentials": {
                    "username": self.username,
                    "apiKey": self.api_key
                }
            }
        })

        headers = {
            'Content-type': 'application/json',
            'Accept': 'application/json'
        }

        response = self.post_fn(self.identity_ep, data=data, headers=headers)

        if response.status_code == 200:

            api_data = response.json()
            self.token = self._parse_token(api_data)

            if self.token:
                self.authenticated = True
                user_regions = self._parse_endpoints(api_data)
                self.regions = self._make_region_list(user_regions)
                self.tenant_id = self._parse_tenant_id(api_data)

        else:
            self.regions = []
            self.token = None

        response.connection.close()

        return self.authenticated

    def list_regions(self):
        """ Return a list the regions available to the user. """

        return self.regions

    def list_flavors(self):
        """ Return the dictionary containing flavors id and names """

        return self._flavors

    def _parse_endpoints(self, api_data):
        """
        Parse the JSON-encoded data returned by the Identity Service API.

        Return a list of regions available for Compute v2.

        """

        region_codes = []

        for ep_type in api_data['access']['serviceCatalog']:
            if ep_type['name'] == "cloudServersOpenStack" \
                    and ep_type['type'] == "compute":

                for ep in ep_type['endpoints']:
                    if ep['versionId'] == "2":
                        region_codes.append(ep['region'])

        return region_codes

    def _parse_token(self, api_data):
        """ Parse the token from the JSON-encoded data returned by the API. """

        try:
            token = api_data['access']['token']['id']
        except KeyError:
            return None

        return token

    def _parse_tenant_id(self, api_data):
        """  """
        try:
            roles = api_data['access']['user']['roles']
            for role in roles:
                if 'tenantId' in role and role['name'] == 'compute:default':
                    return role['tenantId']
            return None
        except KeyError:
            return None

    def _make_region_list(self, region_codes):
        """
        Make a list of regions for use in the GUI.

        Returns a list of key-value pairs in the form:
            <API's Region Name>: <libcloud's Region Name>
            eg,
            [
                {'DFW': 'dfw'}
                {'ORD': 'ord'},
                ...
            ]

        """

        region_list = []

        for ep in ENDPOINT_ARGS_MAP:
            if ENDPOINT_ARGS_MAP[ep]['region'] in region_codes:
                region_list.append({ENDPOINT_ARGS_MAP[ep]['region']: ep})

        return region_list

    def set_region(self, region):
        """ Set self.region and self.driver. Returns True or False. """

        try:
            self.driver = self.driver_cls(self.username, self.api_key,
                                          region=region)
            self.storage_driver = self.storage_driver_cls(self.username, self.api_key,
                                                          region=region)

        except ValueError:
            return False

        self.region = region
        return True

    def get_image(self, image_id):
        return self.driver.get_image(image_id)


def get_provider(cloud_settings):
    """
    Utility function to retrieve a cloud provider instance already authenticated and with the
    region set

    :param cloud_settings: cloud settings dictionary
    :return: a provider instance or None on errors
    """
    try:
        username = cloud_settings['cloud_user_name']
        apikey = cloud_settings['cloud_api_key']
        region = cloud_settings['cloud_region']
    except KeyError as e:
        log.error("Unable to create cloud provider: {}".format(e))
        return

    provider = RackspaceCtrl(username, apikey)

    if not provider.authenticate():
        log.error("Authentication failed for cloud provider")
        return

    if not region:
        region = provider.list_regions().values()[0]

    if not provider.set_region(region):
        log.error("Unable to set cloud provider region")
        return

    return provider



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Interface for VDE (Virtual Distributed Ethernet) NIOs (Unix based OSes only).
"""

from .nio import NIO


class NIOVDE(NIO):

    """
    VDE NIO.

    :param control_file: VDE control filename
    :param local_file: VDE local filename
    """

    def __init__(self, control_file, local_file):

        NIO.__init__(self)
        self._control_file = control_file
        self._local_file = local_file

    def __str__(self):

        return "NIO_VDE"

    def controlFile(self):
        """
        Returns the VDE control file.

        :returns: VDE control filename
        """

        return self._control_file

    def localFile(self):
        """
        Returns the VDE local file.

        :returns: VDE local filename
        """

        return self._local_file



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/news_dock_widget.ui'
#
# Created: Tue Nov 11 15:47:57 2014
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)


class Ui_NewsDockWidget(object):

    def setupUi(self, NewsDockWidget):
        NewsDockWidget.setObjectName(_fromUtf8("NewsDockWidget"))
        NewsDockWidget.resize(203, 225)
        NewsDockWidget.setFloating(False)
        NewsDockWidget.setFeatures(QtGui.QDockWidget.DockWidgetFloatable | QtGui.QDockWidget.DockWidgetMovable)
        NewsDockWidget.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)
        self.dockWidgetContents = QtGui.QWidget()
        self.dockWidgetContents.setObjectName(_fromUtf8("dockWidgetContents"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setSizeConstraint(QtGui.QLayout.SetFixedSize)
        self.horizontalLayout.setContentsMargins(1, 0, 2, 0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.uiWebView = QtWebKit.QWebView(self.dockWidgetContents)
        self.uiWebView.setMinimumSize(QtCore.QSize(200, 200))
        self.uiWebView.setMaximumSize(QtCore.QSize(200, 200))
        self.uiWebView.setProperty("url", QtCore.QUrl(None))
        self.uiWebView.setObjectName(_fromUtf8("uiWebView"))
        self.horizontalLayout.addWidget(self.uiWebView)
        NewsDockWidget.setWidget(self.dockWidgetContents)

        self.retranslateUi(NewsDockWidget)
        QtCore.QMetaObject.connectSlotsByName(NewsDockWidget)

    def retranslateUi(self, NewsDockWidget):
        NewsDockWidget.setWindowTitle(_translate("NewsDockWidget", "Jungle Newsfeed", None))

from PyQt4 import QtWebKit
